using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using SpringCardAPI;

namespace apduxchg_vcs2005
{
    public partial class ApduXchg : Form
    {
        private uint SCardContext;
		private uint SCardHandle;
		private uint SCardProtocol;	// Protocol used by the current card: T0= 1, T1= 2
		private uint rc;
		string strReader;

		private Timer timer1;
		private const int TIMER_INTERVAL= 8000;
		
		private bool reader_active = false;
		private bool card_active = false;

        public ApduXchg()
        {
            timer1 = new System.Windows.Forms.Timer();
			timer1.Interval= TIMER_INTERVAL;
			timer1.Tick += new System.EventHandler(timer1_Tick);

            InitializeComponent();
        }

        #region Conversion (StringToCharArray / HexArrayToString)
        public char [] StringToCharArray (string str)
		{
			char [] res;
			int  len;

			// Check parameter:
			if (str == null)
				return null;

			len= str.Length;
			res= new char [len + 1];

			Array.Copy (str.ToCharArray(), 0, res, 0, len);

			res[len]= '\0';

			return res;
		}

		/*********************************************/

		public string HexArrayToString (byte [] hex, uint len)
		{
			uint  i;
			char [] array;
			byte b1= 0;
			byte b2= 0;

			// Check parameter:
			if (hex == null)
				return null;

			array= new char [ (len*2) + 1];

			for (i=0; i<len; i++)
			{
				b1 = hex[i];
				b2 = b1;

				b1 &= 0x0F;
				b2>>= 4;
			
				if (b1 < 10)
					b1+= 48;
				else
					b1+= 55;

				if (b2 < 10)
					b2+= 48;
				else
					b2+= 55;

				array[2*i]  = (char) b2;
				array[2*i+1]= (char) b1;
			}
			array [len*2]= '\0';

			return new string (array);
        }
        #endregion

        #region Open and Close Reader
        private bool ReaderOpen()
		{
			char [] SCardReaders = new char [1024];
			uint len;

			textBoxInput .Text= "";
			textBoxOutput.Text= "";
		
			// 1) Establish a context:
			rc= SCARD.EstablishContext (SCARD.SCARD_SCOPE_SYSTEM, null, null, ref SCardContext);
			if (rc != SCARD.SCARD_S_SUCCESS)
				return false;

			// 2) Get the list off the available readers:
			len= 1024;
			rc= SCARD.ListReaders (SCardContext, null, SCardReaders, ref len);
			textBox3.Text= SCARD.ErrorToString (SCardContext, rc);
			if (rc != SCARD.SCARD_S_SUCCESS)
				return false;

			// 3) Get the name of the reader corresponding to the specified slot:
			strReader= SCARD.MultiStringItem (SCardContext, SCardReaders, 0);
			if (strReader == null)
				return false;

			// OK:
			textBoxOutput.Text= strReader;

			return true;
		}

		/*********************************************/

		private void ReaderClose ()
		{
			timer1.Enabled = false;
			
			textBoxInput .Text= "";
			textBoxOutput.Text= "";

			SCARD.ReleaseContext (SCardContext);
			textBoxOutput.Text= "Reader closed";
		}
        #endregion

        #region Card communication (Connect / Status / Exchange / Close)
        private bool CardConnect ()
		{
			uint  cardState= 0;
			byte [] atr= new byte [SCARD.SCARD_ATR_LENGTH];
			uint  len;
			uint  i= 0;
			uint  protoFlag;

			textBoxInput .Text= "";
			textBoxOutput.Text= "";


			// 1) Connect to the card:
			protoFlag= SCARD.SCARD_PROTOCOL_Tx;
			if (checkBoxPPS.Checked)
				protoFlag|= SCARD.SCARD_PROTOCOL_PTS;
			if (checkBoxEMV.Checked)
				protoFlag|= SCARD.SCARD_PROTOCOL_EMV;
			
			rc= SCARD.Connect (
				SCardContext,
				StringToCharArray (strReader),
				SCARD.SCARD_SHARE_EXCLUSIVE, 
				protoFlag,
				ref SCardHandle,
				ref SCardProtocol
				);
			textBox3.Text= SCARD.ErrorToString (SCardContext, rc);
		
			if (rc != SCARD.SCARD_S_SUCCESS)
			{
				textBoxOutput.Text= "CONNECT FAILED";
				return false;
			}


			// 2) Get card's ATR:
			len= SCARD.SCARD_ATR_LENGTH;
			rc = SCARD.Status (SCardHandle, null, ref i, ref cardState, ref SCardProtocol, atr, ref len);
			textBox3.Text= SCARD.ErrorToString (SCardContext, rc);
			if (rc != SCARD.SCARD_S_SUCCESS)
			{
				textBoxOutput.Text= "STATUS FAILED";
				return false;
			}

			
			// OK:
			textBoxOutput.Text= HexArrayToString (atr, len);

			timer1.Enabled = true;

			return true;
		}

		/*********************************************/

		private bool CardStatus ()
		{
			uint  cardState= 0;
			byte [] atr= new byte [SCARD.SCARD_ATR_LENGTH];
			uint  len;
			uint  i= 0;

			len= SCARD.SCARD_ATR_LENGTH;
			rc = SCARD.Status (SCardHandle, null, ref i, ref cardState, ref SCardProtocol, atr, ref len);
			if (rc != SCARD.SCARD_S_SUCCESS)
			{
				textBoxOutput.Text= "STATUS FAILED";
				return false;
			}

			return true;
		}
		
        /*********************************************/

		private bool CardClose ()
		{
			textBoxInput .Text= "";
			textBoxOutput.Text= "";

			timer1.Enabled = false;

			rc= SCARD.Disconnect (SCardHandle, SCARD.SCARD_UNPOWER_CARD);
			textBox3.Text= SCARD.ErrorToString (SCardContext, rc);
			if (rc != SCARD.SCARD_S_SUCCESS)
			{
				textBoxOutput.Text= "CLOSE_FAILED";
				return false;
			}
			textBoxOutput.Text= "CLOSE_OK";
			return true;
		}
		
        /*********************************************/

		public bool ExchangeAPDU ()
		{
			uint  APDULen;
			byte [] APDU;
			byte [] buffer= new byte [500];
			uint  len;
	
			string strAPDU= textBoxInput.Text;

			APDULen= (uint) strAPDU.Length/2;
			APDU   = new byte [APDULen];

			// Convert APDU string into an array of bytes:
			try
			{
				for (int i=0; i<APDULen; i++)
					APDU[i]= byte.Parse (strAPDU.Substring(i*2, 2), System.Globalization.NumberStyles.AllowHexSpecifier);
			}
			catch (Exception)
			{
				MessageBox.Show ("Invalid APDU", "Error");
				return false;
			}

			// Rewrite APDU if different:
			textBoxInput.Text= HexArrayToString (APDU, APDULen);

			
			len= 500;
			rc= SCARD.Transmit (SCardHandle, APDU, APDULen, buffer, ref len);
			textBox3.Text= SCARD.ErrorToString (SCardContext, rc);
			if (rc != SCARD.SCARD_S_SUCCESS)
			{
				textBoxOutput.Text= "TRANSMIT FAILED";
				return false;
			}
		
			textBoxOutput.Text= HexArrayToString (buffer, len);

			return true;
        }
        #endregion

        #region Action (Click / Tick)

        private void ActivateButtons()
        {
            buttonOpen.Enabled = !reader_active;
            buttonClose.Enabled = reader_active;
            buttonOpen.Visible = !reader_active;
            buttonClose.Visible = reader_active;
            buttonConnect.Enabled = reader_active && !card_active;
            buttonDisconnect.Enabled = reader_active && card_active;
            buttonConnect.Visible = !card_active;
            buttonDisconnect.Visible = card_active;
            buttonSend.Enabled = reader_active && card_active;
            checkBoxPPS.Enabled = !card_active;
            checkBoxEMV.Enabled = !card_active;
            checkBoxSAM.Enabled = !reader_active;
        }

        private void buttonOpen_Click(object sender, System.EventArgs e)
		{
			Cursor.Current= Cursors.WaitCursor;
			reader_active = ReaderOpen();
			Cursor.Current= Cursors.Default;
			ActivateButtons();
		}

		/*********************************************/

		private void buttonClose_Click(object sender, System.EventArgs e)
		{
			Cursor.Current= Cursors.WaitCursor;
			ReaderClose ();
			reader_active = false;
			card_active = false;
			Cursor.Current= Cursors.Default;
			ActivateButtons ();
		}

		/*********************************************/

		private void buttonConnect_Click(object sender, System.EventArgs e)
		{
			Cursor.Current = Cursors.WaitCursor;
			card_active = CardConnect();
			Cursor.Current= Cursors.Default;
			ActivateButtons ();
		}

		/*********************************************/

		private void buttonDisconnect_Click(object sender, System.EventArgs e)
		{
			Cursor.Current=Cursors.WaitCursor;
			CardClose();
			card_active = false;
			Cursor.Current= Cursors.Default;
			ActivateButtons ();
		}

		/*********************************************/

		private void buttonSend_Click(object sender, System.EventArgs e)
		{
			timer1.Enabled = false;		
			Cursor.Current=Cursors.WaitCursor;
			ExchangeAPDU();
			Cursor.Current= Cursors.Default;
			timer1.Enabled = true;			
		}

		/*********************************************/

		private void timer1_Tick(object sender, System.EventArgs e)
		{
			if (!CardStatus())
				timer1.Enabled = false;
        }
        #endregion

        private void textBoxInput_GotFocus(object sender, EventArgs e)
        {
            inputPanel.Enabled = true;
        }

        private void textBoxInput_LostFocus(object sender, EventArgs e)
        {
            inputPanel.Enabled = false;
        }

    }
}