/*
  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.

  SpringCard Library
  ------------------

  Copyright (c) 2000-2008 SpringCard - www.springcard.com
  See COPYING for copyright and legal info

  config_win32.c
  --------------
  Where to store configuration settings under Win32

  revision :
  ----------

	JDA 09/04/2004 : created
	JDA 23/10/2006 : added the ability to put DeviceName in registry
  LTC 17/12/2008 : change HKEY_SPRINGCARD to "_T("Software\\SpringCard\\SpringCard")"
*/

#include "springcard_i.h"

#ifdef WIN32

#include "config_win32.h"

#define HKEY_SPRINGCARD       _T("Software\\SpringCard\\SpringCard")
#define SPRINGCARD_PNPID      _T("PRO-ACTIVE_FR-SPRINGCARD")
#define ARASAN_DEVICE_KEY_STR _T("\\Drivers\\SDCARD\\ClientDrivers\\Custom\\MANF-0296-CARDID-5347-FUNC-1")

static BOOL GetDefaultDeviceFromRegistry(TCHAR *device, WORD len, DWORD *device_specific_info);
static BOOL GetDefaultDeviceFromHardware(TCHAR *device, WORD len, DWORD *device_specific_info);

BOOL GetDefaultDevice(TCHAR *device, WORD len, DWORD *device_specific_info)
{
  if (GetDefaultDeviceFromHardware(device, len, device_specific_info)) return TRUE;
  if (GetDefaultDeviceFromRegistry(device, len, device_specific_info)) return TRUE;
  return FALSE;
}

static BOOL GetDefaultDeviceFromRegistry(TCHAR *device, WORD len, DWORD *device_specific_info)
{
  HKEY    hKey;
  DWORD   size;

  /* Open the HKCU\Sofware\SpringCard\SpringCard key in registry */
  if (RegOpenKeyEx(HKEY_CURRENT_USER, HKEY_SPRINGCARD, 0, KEY_READ|KEY_EXECUTE, &hKey) == ERROR_SUCCESS)
  {
    /* Get subkeys count */
    size = len * sizeof(TCHAR);
    if (RegQueryValueEx(hKey, _T("DeviceName"), NULL, NULL, (LPBYTE) device, &size) == ERROR_SUCCESS)
    {
      /* Found ! */
      RegCloseKey(hKey);
      return TRUE;
    }
    RegCloseKey(hKey);
  }

  /* Open the HKLM\Sofware\SpringCard\SpringCard key in registry */
  if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, HKEY_SPRINGCARD, 0, KEY_READ|KEY_EXECUTE, &hKey) == ERROR_SUCCESS)
  {
    /* Get subkeys count */
    size = len * sizeof(TCHAR);
    if (RegQueryValueEx(hKey, _T("DeviceName"), NULL, NULL, (LPBYTE) device, &size) == ERROR_SUCCESS)
    {
      /* Found ! */
      RegCloseKey(hKey);
      return TRUE;
    }
    RegCloseKey(hKey);
  }

  return FALSE;
}

static BOOL GetDefaultDeviceFromHardware(TCHAR *device, WORD len, DWORD *device_specific_info)
{
  DWORD   i;

  HKEY    hKey, hSubKey;
  DWORD   size;
  TCHAR   shortBuffer[32];
  TCHAR   longBuffer[256];

  if (device_specific_info != NULL)
    *device_specific_info = 0x00000000;

  /* Open the HKLM\Drivers\Active key in registry */
  if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, _T("Drivers\\Active"), 0, 0, &hKey) == ERROR_SUCCESS)
  {
    /* Get subkeys count */
    if (RegQueryInfoKey(hKey, NULL, NULL, NULL, &i, NULL, NULL, NULL, NULL, NULL, NULL, NULL) == ERROR_SUCCESS)
    {
      /* Browse subkeys in reverse order (pluggable cards are not at the beginning of the list !) */
      while (i)
      {
        i--;
        /* Select the subkey */
        size = sizeof(shortBuffer);
        if (RegEnumKeyEx(hKey, i, shortBuffer, &size, NULL, NULL, NULL, NULL) == ERROR_SUCCESS)
        {
          /* Open the subkey */
          shortBuffer[sizeof(shortBuffer) - 1] = '\0';
          _stprintf(longBuffer, _T("Drivers\\Active\\%s"), shortBuffer);
          if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, longBuffer, 0, 0, &hSubKey) == ERROR_SUCCESS)
          {
            /* Read the PnPID value */
            size = sizeof(longBuffer);
            if (RegQueryValueEx(hSubKey, _T("PnPID"), NULL, NULL, (LPBYTE) longBuffer, &size) == ERROR_SUCCESS)
            {
              /* Compare the PnPID to requested one */
              if (_tcsncmp(longBuffer, SPRINGCARD_PNPID, _tcsclen(SPRINGCARD_PNPID)) == 0)
              {
                /* This is the good PnPID, now get the serial com in the "Name" value */
                size = len;
                if (RegQueryValueEx(hSubKey, _T("Name"), NULL, NULL, (LPBYTE) longBuffer, &size) == ERROR_SUCCESS)
                {
                  /* Found ! */
                  _tcsncpy(device, longBuffer, len);
                  /* Done with the registry ! */
                  RegCloseKey(hSubKey);
                  RegCloseKey(hKey);
                  /* OK */
                  if (device_specific_info != NULL)
                    *device_specific_info |= DEVICE_INFO_TYPE_CF; /* CompactFlash device */
                  return TRUE;
                }
              }
            }

            /* New 1.20 (for SpringXXXX-SD) : lookup for Arasan SIO1: device */
            size = sizeof(longBuffer);
            if (RegQueryValueEx(hSubKey, _T("Key"), NULL, NULL, (LPBYTE) longBuffer, &size) == ERROR_SUCCESS)
            {
              if (_tcsncmp(longBuffer, ARASAN_DEVICE_KEY_STR, _tcsclen(ARASAN_DEVICE_KEY_STR)) == 0)
              {
                /* This is the good PnPID, now get the serial com in the "Name" value */
                size = len;
                if (RegQueryValueEx(hSubKey, _T("Name"), NULL, NULL, (LPBYTE) longBuffer, &size) == ERROR_SUCCESS)
                {
                  /* Found ! */
                  _tcsncpy(device, longBuffer, len);
                  /* Done with the registry ! */
                  RegCloseKey(hSubKey);
                  RegCloseKey(hKey);
                  /* OK */
                  if (device_specific_info != NULL)
                    *device_specific_info |= DEVICE_INFO_TYPE_SD; /* SDIO device */
                  return TRUE;
                }
              }
            }

            /* Close the subkey */
            RegCloseKey(hSubKey);
          }
        }
      }
    }
  }
  /* Close the key */
  RegCloseKey(hKey);

  /* Not found */
  return FALSE;
}

void LoadSettings(void)
{
  HKEY    hKey;
  DWORD   size;
  TCHAR   buffer[256];
  BYTE    loop;
  BYTE    level = 0;
  
  for (loop=0; loop<=1; loop++)
  {
    if (loop == 0)
    {
       /* First step : try to locate debug config under HKCU\Sofware\SpringCard\SpringCard */
      if (!RegOpenKeyEx(HKEY_CURRENT_USER, HKEY_SPRINGCARD, 0, KEY_READ|KEY_EXECUTE, &hKey) == ERROR_SUCCESS)
        continue;
    } else
    {
      /* Second step : do the same under HKLM\Sofware\SpringCard\SpringCard */
      if (!RegOpenKeyEx(HKEY_LOCAL_MACHINE, HKEY_SPRINGCARD, 0, KEY_READ|KEY_EXECUTE, &hKey) == ERROR_SUCCESS)
        continue;
    }
    
    size = sizeof(buffer);
    if (RegQueryValueEx(hKey, _T("DebugLevel"), NULL, NULL, (LPBYTE) buffer, &size) == ERROR_SUCCESS)
      level = (BYTE) _ttoi(buffer);

    size = sizeof(buffer);        
    if (RegQueryValueEx(hKey, _T("DebugFile"), NULL, NULL, (LPBYTE) buffer, &size) == ERROR_SUCCESS)   
      SCARD_TraceSetFile(buffer);

    RegCloseKey(hKey);
  }
  
  if (level)
  {
    SCARD_TraceSetLevel(level);
    
    SCARD_Trace(TR_COMM_LOW, "LoadSettings: SPRINGCARD SPRINGCARD DLL %X.%02X [%d]", VERSION_MAJOR, VERSION_MINOR, VERSION_BUILD);
    
    if (level == 0xFF)
    {
      MessageBox(0, _T("The DLL is currently running with highest debug level. Be careful that debug file will become rapidly very very big !"), _T("SpringCard API"), 0);      
    }
  }
}

BOOL APIENTRY DllMain(HANDLE hModule, DWORD ul_reason_for_call, LPVOID lpReserved)
{
  __try
  {
    switch (ul_reason_for_call)
    {
      case DLL_PROCESS_ATTACH:
  			LoadSettings();
        break;
      case DLL_THREAD_ATTACH:
        break;
      case DLL_THREAD_DETACH:
        break;
      case DLL_PROCESS_DETACH:
        break;
    }
    return TRUE;
  }
  __except(1)
  {
    return FALSE;
  }
}

#endif
