/*
  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.

  Copyright (c) 2000-2008 SpringCard - www.springcard.com
  See COPYING for copyright and legal info

  pssc_low.c
  ----------
  Low level functions - GBP API wrappers

 */

#include "springcard_i.h"

/*
 * Dirty direct transmission
 * -------------------------
 */
LONG GCR_Command(SCARD_HANDLE_ST *card, BYTE *sendBuffer, DWORD sendLen, BYTE *recvBuffer, DWORD recvSize, DWORD *recvLen, DWORD timeout)
{
  ReaderDesc *rdr;

  if (sendBuffer == NULL)
    return SCARD_E_INVALID_PARAMETER;

  if (sendLen >= GBP_MAX_CMD_LEN) //CMD_BUF_SIZE
    return SCARD_E_INVALID_PARAMETER;

  if ((recvBuffer == NULL) && (recvSize != 0))
    return SCARD_E_INVALID_PARAMETER;

  rdr = &((SCARD_CONTEXT_ST *) card->context)->desc;
  rdr->cmd_timeout = 2*timeout; // JDA : TODO

  return GBP_Command(rdr, sendLen, sendBuffer, recvLen, recvBuffer);
}

/*
 * Outgoing APDU
 * -------------
 */
static LONG GCR_IsoOut(SCARD_HANDLE_ST *card, BYTE *sendBuffer, DWORD sendLen, BYTE *recvBuffer, DWORD recvSize, DWORD *recvLen)
{
  BYTE    cmdBuffer[GCR_SEND_BUF_SIZE];
  LONG    rc;

  /* Buffer only holds an APDU (no Data), */
  /* output command                       */

  memset(recvBuffer, 0xDD, recvSize);

  if (card->slot == GCR_SLOT_0)
    cmdBuffer[GC_OFFSET_CMD] = GCR_ISO_OUT;
  else if (card->slot == GCR_SLOT_1)
    cmdBuffer[GC_OFFSET_CMD] = GCR_ISO_OUT_AUX;
  else
  {
    SCARD_Trace(TR_GCR_HIGH, "No slot %d", card->slot);
    return SCARD_E_INVALID_HANDLE;
  }

  memcpy(cmdBuffer + GC_OFFSET_APDU, sendBuffer, sendLen);

  /* Send the command */
  *recvLen = 0;
  rc = GCR_Command(card, cmdBuffer, sendLen + GC_SIZE_CMD, recvBuffer, recvSize, recvLen, card->timeout);
  if (rc != SCARD_S_SUCCESS)
  {
    SCARD_Trace(TR_GCR_HIGH, "ISO Output failed (%08lX)", rc);
    return rc;
  }

  if (*recvLen == 253)
  {
    /* More data to follow */
    DWORD moreSize, moreLen;

    memset(&cmdBuffer[1], 0xFF, 5);

    moreSize = *recvLen - 252;
    moreLen  = 0;

    rc = GCR_Command(card, cmdBuffer, 6, &recvBuffer[253], moreSize, &moreLen, 500);
    if (rc != SCARD_S_SUCCESS)
    {
      SCARD_Trace(TR_GCR_HIGH, "ISO Output failed (%08lX)", rc);
      return rc;
    }

    *recvLen += moreLen;
  }

  /* OK */
  return SCARD_S_SUCCESS;
}

/*
 * Incoming APDU
 * -------------
 */
static LONG GCR_IsoIn(SCARD_HANDLE_ST *card, BYTE *sendBuffer, DWORD sendLen, BYTE *recvBuffer, DWORD recvSize, DWORD *recvLen)
{
  BYTE    cmdBuffer[GCR_SEND_BUF_SIZE];
  LONG    rc;

  /* Buffer only holds an APDU (no Data), */
  /* output command                       */

  if (card->slot == GCR_SLOT_0)
    cmdBuffer[GC_OFFSET_CMD] = GCR_ISO_IN;
  else if (card->slot == GCR_SLOT_1)
    cmdBuffer[GC_OFFSET_CMD] = GCR_ISO_IN_AUX;
  else
  {
    SCARD_Trace(TR_GCR_HIGH, "No slot %d", card->slot);
    return SCARD_E_INVALID_HANDLE;
  }

  if (sendLen > 248 + 5)
  {
    /* Prefix */
    memset(&cmdBuffer[1], 0xFF, 4);
    cmdBuffer[5] = (BYTE) (sendLen - 248 - 5);
    memcpy(&cmdBuffer[6], &sendBuffer[248 + 5], cmdBuffer[5]);

    rc = GCR_Command(card, cmdBuffer, 6 + cmdBuffer[5], recvBuffer, recvSize, recvLen, 500);
    if (rc != SCARD_S_SUCCESS)
      return rc;
    
    sendLen = 248 + 5;
  }

  memcpy(cmdBuffer + GC_OFFSET_APDU, sendBuffer, sendLen);

  rc = GCR_Command(card, cmdBuffer, sendLen + GC_SIZE_CMD, recvBuffer, recvSize, recvLen, card->timeout);
  if (rc != SCARD_S_SUCCESS)
  {
    SCARD_Trace(TR_GCR_HIGH, "ISO Input failed (%08lX)", rc);
    return rc;
  }

  /* OK */
  return SCARD_S_SUCCESS;
}

/*
 * Exchange APDU (T=1)
 * -------------------
 */
static LONG GCR_Apdu(SCARD_HANDLE_ST *card, BYTE *sendBuffer, DWORD sendLen, BYTE *recvBuffer, DWORD recvSize, DWORD *recvLen)
{
  BYTE    cmdBuffer[GCR_SEND_BUF_SIZE];
  LONG    rc;

  /* Build the commmand */
  if (card->slot == GCR_SLOT_0)
    cmdBuffer[GC_OFFSET_CMD] = GCR_APDU;
  else if (card->slot == GCR_SLOT_1)
    cmdBuffer[GC_OFFSET_CMD] = GCR_APDU_AUX;
  else
  {
    SCARD_Trace(TR_GCR_HIGH, "No slot %d", card->slot);
    return SCARD_E_INVALID_HANDLE;
  }

  if (sendLen > 254)
  {
    /* Prefix */
    memset(&cmdBuffer[1], 0xFF, 4);
    cmdBuffer[5] = (BYTE) (sendLen - 254);
    memcpy(&cmdBuffer[6], &sendBuffer[254], cmdBuffer[5]);

    rc = GCR_Command(card, cmdBuffer, 6 + cmdBuffer[5], recvBuffer, recvSize, recvLen, 500);
    if (rc != SCARD_S_SUCCESS)
      return rc;
    
    sendLen = 254;
  }

  memcpy(cmdBuffer + GC_OFFSET_APDU, sendBuffer, sendLen);

  /* Add length of expected data */
  cmdBuffer[GC_OFFSET_APDU + sendLen] = (BYTE) recvSize;

  rc = GCR_Command(card, cmdBuffer, sendLen + GC_SIZE_CMD, recvBuffer, recvSize, recvLen, card->timeout);
  if (rc == SCARD_F_INTERNAL_ERROR)
  {
    rc = SCARD_S_SUCCESS;
  }
  if (rc != SCARD_S_SUCCESS)
  {    
    SCARD_Trace(TR_GCR_HIGH, "ISO Exchange failed (%08lX)", rc);
    return rc;
  }

  if (*recvLen == 254)
  {
    /* More data to follow */
    DWORD moreSize, moreLen;

    memset(&cmdBuffer[1], 0xFF, 5);

    moreSize = *recvLen - 254;
    moreLen  = 0;

    rc = GCR_Command(card, cmdBuffer, 6, &recvBuffer[254], moreSize, &moreLen, 500);
    if (rc != SCARD_S_SUCCESS)
    {
      SCARD_Trace(TR_GCR_HIGH, "ISO Exchange failed (%08lX)", rc);
      return rc;
    }

    *recvLen += moreLen;
  }

  /* OK */
  return SCARD_S_SUCCESS;
}

/*
 * APDU exchange core
 * ------------------
 */
LONG GCR_Transmit(SCARD_HANDLE_ST *card, BYTE *sendBuffer, DWORD sendLen, BYTE *recvBuffer, DWORD recvSize, DWORD *recvLen)
{
  if (GCR_SEND_BUF_SIZE < 1 + sendLen)
  {
    /* Buffer too small, send an error */
    return SCARD_E_INVALID_VALUE;
  }
  
  switch (card->run_protocol)
  {
    case SCARD_PROTOCOL_RAW:
      if (gcr_family == GCR_LITE)
      {
      	//MessageBox(0, _T("reader_card_apdu 1"), _T("SCARD_E_UNSUPPORTED_FEATURE"), 0);
        return SCARD_E_UNSUPPORTED_FEATURE;
      }
      /* GemCore is not lite, so the T=0 like APDU can be properly interpreted */

#if 0
    case SCARD_PROTOCOL_T0:

      /* Check if this is an incoming or outgoing command */
      /* Size should be command + one byte of length for  */
      /* an outgoing TPDU (CLA, INS, P1, P2, P3)          */
      if (sendLen == 5)
      {
        /* Outgoing APDU */
        return GCR_IsoOut(card, sendBuffer, sendLen, recvBuffer, recvSize, recvLen);
      }

      /* Just (CLA, INS, P1, P2) for an APDU */
      if (sendLen == 4)
      {
        /* Incoming APDU with no data */
        BYTE cmd[5];

        /* copy CLA, INS, P1, P2 */
        memcpy(cmd, sendBuffer, 4);
        /* set P3 to 0 */
        cmd[4] = 0;

        return GCR_IsoIn(card, cmd, 5, recvBuffer, recvSize, recvLen);
      }

      if (sendLen > 5)
      {
        /* Check length to see if it is a full APDU or a TPDU */
        DWORD l;

        l  = sendBuffer[4]; /* value of Lc in P3 */
        if (l == 0)  l = 256;
        l += 5;  /* CLA, INS, P1, P2, P3 */

        if (sendLen == l + 1) /* One more for Le after data */
        {
          /* sendBuffer holds a proper APDU */
          return GCR_Apdu(card, sendBuffer, sendLen, recvBuffer, recvSize, recvLen);
        } else
        if (sendLen == l)
        {
          /* Incoming TPDU */
          return GCR_IsoIn(card, sendBuffer, sendLen, recvBuffer, recvSize, recvLen);
        }
      }

      /* sendBuffer holds too little data to form an APDU+length */
      return SCARD_E_INVALID_VALUE;
      break;

#else
    case SCARD_PROTOCOL_T0:
#endif

    case SCARD_PROTOCOL_T1:
      /* Check if command is going to fit in buffer */
      /* cmd byte + sendLen + Le                    */

      /* T=1 APDU exchange function */
      return GCR_Apdu(card, sendBuffer, sendLen, recvBuffer, recvSize, recvLen);
      break;

    default:
    	//MessageBox(0, _T("reader_card_apdu 2"), _T("SCARD_E_UNSUPPORTED_FEATURE"), 0);
      return SCARD_E_UNSUPPORTED_FEATURE;
  }
}



/*
 * Set mode to ISO or EMV
 * ----------------------
 */
LONG GCR_SetMode(SCARD_HANDLE_ST *card, BYTE mode)
{
  BYTE    command[3];
  BYTE    buffer[32];
  DWORD   recvLen;

  /* Build command */
  if (card->slot == GCR_SLOT_0)
    command[0] = 0x17;
  else if (card->slot == GCR_SLOT_1)
    command[0] = 0x1F;
  else
  {
    SCARD_Trace(TR_GCR_HIGH, "No slot %d", card->slot);
    return SCARD_E_INVALID_HANDLE;
  }
  command[1] = 0x00;
  command[2] = mode;
  
  SCARD_Trace(TR_GCR_HIGH, "Set mode %02X", mode);

  /* Send command */
  return GCR_Command(card, command, sizeof(command), buffer, sizeof(buffer), &recvLen, 0);
}

/*
 * Set type
 * --------
 */
LONG GCR_SetType(SCARD_HANDLE_ST *card, BYTE type)
{
  BYTE    command[2];
  char    buffer[32];
  DWORD   recvLen;

  /* Build command */
  if (card->slot == GCR_SLOT_0)
    command[0] = 0x17;
  else if (card->slot == GCR_SLOT_1)
    command[0] = 0x1F;
  else
  {
    SCARD_Trace(TR_GCR_HIGH, "No slot %d", card->slot);
    return SCARD_E_INVALID_HANDLE;
  }
  command[1] = type;

  // TODO
  if (card->slot == GCR_SLOT_1)
    return SCARD_S_SUCCESS;
    
  SCARD_Trace(TR_GCR_HIGH, "Set type %02X", type);

  /* Send command */
  return GCR_Command(card, command, sizeof(command), buffer, sizeof(buffer), &recvLen, 0);
}

/*
 * Get firmware
 * ------------
 */
LONG GCR_GetFirmware(ReaderDesc * reader, BYTE buffer[], DWORD * length)
{
  BYTE cmd[] = { 0x22, 0x05, 0x3F, 0xE0, 0x10 };

  if (*length < IFD_LEN_VERSION)
    return SCARD_E_INSUFFICIENT_BUFFER;

  /* clear the buffer to be sure it is \0 terminated */
  memset(buffer, 0, *length);
  /* length-1 to be sure their is room for the \0 */
  *length = (*length) - 1;
  
  return GBP_Command(reader, sizeof(cmd), cmd, length, buffer);
}

LONG GCR_SetBaudrate(ReaderDesc * reader, DWORD baudrate)
{

  BYTE    cmd[2];
  BYTE    buffer[32];
  DWORD   length;

  cmd[0] = 0x0A;
  switch (baudrate)
  {
    case 76800 : cmd[1] = 0x01; break;
    case 38400 : cmd[1] = 0x02; break;
    case 19200 : cmd[1] = 0x03; break;
    case 9600  : cmd[1] = 0x04; break;
    default    : return SCARD_E_INVALID_VALUE;
  }
  
  length = sizeof(buffer);
  GBP_Command(reader, sizeof(cmd), cmd, &length, buffer);
  
  SerialFlush(reader->fd);
  SerialConfig(reader->fd, baudrate); 
  SerialFlush(reader->fd);  

  return SCARD_S_SUCCESS;
}

/*
 * Read card status
 * ----------------
 */
LONG GCR_Status(SCARD_HANDLE_ST *card)
{
  BYTE    command[1];
  BYTE    buffer[32];
  LONG    rc;
  DWORD   recvLen;

  /* Build command */
  if (card->slot == GCR_SLOT_0)
    command[0] = 0x17;
  else if (card->slot == GCR_SLOT_1)
    command[0] = 0x1F;
  else
  {
    SCARD_Trace(TR_GCR_HIGH, "No slot %d", card->slot);
    return SCARD_E_INVALID_HANDLE;
  }

  /* Send command */
  rc = GCR_Command(card, command, sizeof(command), buffer, sizeof(buffer), &recvLen, 0);

  if (rc != SCARD_S_SUCCESS)
  {
    /* Failed... */
    SCARD_Trace(TR_GCR_HIGH, "Get status failed (%08lX)", rc);
    card->status = 0;
    return rc;
  }

  /* Decode response */
  /* --------------- */
  
  SCARD_Trace(TR_GCR_HIGH, "Status = %02X %02X", buffer[0], buffer[1]);

  /* Card status */
  card->run_protocol = 0;

  if (buffer[0] & CARD_INSERTED)
  {
    if (buffer[0] & CARD_POWERED)
    {
      card->status = SCARD_POWERED; /* Power is being provided to the card, but the reader driver is unaware of the mode of the card. */

      /* Card type */
      if ((buffer[1] == 0x02) || (buffer[1] == 0x12))
      {
        /* T=0 or T=1 */
        if (buffer[0] & T1_PROTOCOL)
        {
          if (buffer[0] & POWER_SUPPLY_3V)
          {
            SCARD_Trace(TR_GCR_HIGH, "Card powered on, 3V, T=1");
          } else
          {
            SCARD_Trace(TR_GCR_HIGH, "Card powered on, 5V, T=1");
          }
          card->run_protocol = SCARD_PROTOCOL_T1;
        } else
        {
          if (buffer[0] & POWER_SUPPLY_3V)
          {
            SCARD_Trace(TR_GCR_HIGH, "Card powered on, 3V, T=0");
          } else
          {
            SCARD_Trace(TR_GCR_HIGH, "Card powered on, 5V, T=0");
          }
          card->run_protocol = SCARD_PROTOCOL_T0;
        }
      } else
      {
      	if (buffer[0] & POWER_SUPPLY_3V)
      	{
        	SCARD_Trace(TR_GCR_HIGH, "Card powered on, 3V, not ISO");
        } else
        {
        	SCARD_Trace(TR_GCR_HIGH, "Card powered on, 5V, not ISO");
        }
        card->run_protocol = SCARD_PROTOCOL_RAW;
      }
    } else
    {
      SCARD_Trace(TR_GCR_HIGH, "Card swallowed");
      card->status = SCARD_SWALLOWED; /* There is a card in the reader in position for use. The card is not powered. */
    }
  } else
  {
    SCARD_Trace(TR_GCR_HIGH, "Card absent");
    card->status = SCARD_ABSENT;  /* There is no card in the reader */
  }

  return SCARD_S_SUCCESS;
}

/*
 * Unpower the card
 * ----------------
 */
LONG GCR_PowerDown(SCARD_HANDLE_ST *card)
{
  BYTE    command[1];
  BYTE    buffer[32];
  DWORD   recvLen;

  /* Build command */
  if (card->slot == GCR_SLOT_0)
    command[0] = 0x11;
  else if (card->slot == GCR_SLOT_1)
    command[0] = 0x19;
  else
  {
    SCARD_Trace(TR_GCR_HIGH, "No slot %d", card->slot);
    return SCARD_E_INVALID_HANDLE;
  }

  SCARD_Trace(TR_GCR_HIGH, "Power down");

  /* Send command */
  return GCR_Command(card, command, sizeof(command), buffer, sizeof(buffer), &recvLen, 0);
}

/*
 * Power-up the card
 * -----------------
 */
LONG GCR_PowerUpEx(SCARD_HANDLE_ST *card, BYTE cfg)
{
  BYTE    command[2];
  BYTE    buffer[48];
  LONG    rc;
  DWORD   recvLen;

  /* Build command */
  if (card->slot == GCR_SLOT_0)
    command[0] = 0x12;
  else if (card->slot == GCR_SLOT_1)
    command[0] = 0x1A;
  else
  {
    SCARD_Trace(TR_GCR_HIGH, "No slot %d", card->slot);
    return SCARD_E_INVALID_HANDLE;
  }
  command[1] = cfg;
  
  SCARD_Trace(TR_GCR_HIGH, "Power up %02X", cfg);

  rc = GCR_Command(card, command, sizeof(command), buffer, sizeof(buffer), &recvLen, 0);
  if (rc != SCARD_S_SUCCESS)
  {
    SCARD_Trace(TR_GCR_HIGH, "Power up failed (%08lX)", rc);
    return rc;
  }

  /* This is the ATR */
  if (recvLen > MAX_ATR_SIZE)
    recvLen = MAX_ATR_SIZE;
  card->atr_len = (BYTE) recvLen; /* ATR theory !!! */
  memcpy(card->atr, buffer, card->atr_len);

  /* Card is now running ! */
  return SCARD_S_SUCCESS;
}

/*
 * Try to configure and to power-up the card
 * -----------------------------------------
 */
LONG GCR_PowerUp(SCARD_HANDLE_ST *card)
{
  BYTE    command[1];
  BYTE    buffer[48];
  LONG    rc;
  DWORD   recvLen;

  /* Build command */
  if (card->slot == GCR_SLOT_0)
    command[0] = 0x12;
  else if (card->slot == GCR_SLOT_1)
    command[0] = 0x1A;
  else
  {
    SCARD_Trace(TR_GCR_HIGH, "No slot %d", card->slot);
    return SCARD_E_INVALID_HANDLE;
  }

  SCARD_Trace(TR_GCR_HIGH, "Power up (short)");

  rc = GCR_Command(card, command, sizeof(command), buffer, sizeof(buffer), &recvLen, 0);
  if (rc != SCARD_S_SUCCESS)
  {
    SCARD_Trace(TR_GCR_HIGH, "Power up failed (%08lX)", rc);
    return rc;
  }

  /* This is the ATR */
  card->atr_len = (BYTE) recvLen; /* ATR theory !!! */
  if (card->atr_len > MAX_ATR_SIZE)
    card->atr_len = MAX_ATR_SIZE;

  memcpy(card->atr, buffer, card->atr_len);

  /* Card is now running ! */
  return SCARD_S_SUCCESS;
}

