
/*
 *  GCCmds.c
 *  $Id: GCCmds.c,v 1.16 2002/09/18 19:53:16 rousseau Exp $
 *  ifd-GemPC
 *
 *  Created by Jean-Luc Giraud on Sat Oct 20 2001.
 *  Updated by Ludovic Rousseau, Oct 2001.
 *
 *  Copyright (c) 2001 Jean-Luc Giraud, Ludovic Rousseau
 *
 *  License: this code is under a double licence COPYING.BSD and COPYING.GPL
 *
 */

#include "springcard_i.h"
#include "pcsc_impl.h"
#include "gbp_impl.h"

#include <stdio.h>
#include <string.h>

static LONG GBP_TranslateStatus(BYTE gcr_status);

static LONG GBP_CommandEx(ReaderDesc * reader, DWORD nLengthIn, const BYTE pcBufferCmd[], DWORD * pnLengthOut, BYTE pcBufferOut[], BYTE * gcr_resp)
{
  LONG    rc;
  BYTE    rv;
  DWORD   lengthout;
  BYTE    buffer[GC_TR_BUF_SIZE];

  if (SCARD_TraceGetLevel() & TR_GCR_LOW)
  {
    DWORD i;
    
    SCARD_Trace(TR_GCR_LOW, "Command> %02X ", pcBufferCmd[0]);
    for (i=1; i<nLengthIn; i++)
      SCARD_Trace(TR_GCR_LOW, "_%02X", pcBufferCmd[i]);
  }

  lengthout = sizeof(buffer);
  rc = GBPSendCommand(reader, nLengthIn, pcBufferCmd, &lengthout, buffer);

  // transport level error
  if (rc != SCARD_S_SUCCESS)
    return rc;

  // GemCore level error
  rv = buffer[GC_STATUS_OFFSET];

  if (SCARD_TraceGetLevel() & TR_GCR_LOW)
  {
    DWORD i;
    
    SCARD_Trace(TR_GCR_LOW, "Command< %02X ", rv);
    for (i=1; i<lengthout; i++)
      SCARD_Trace(TR_GCR_LOW, "_%02X", buffer[i]);
  }

  // shift command anwser (remove GemCore status byte)
  if (pcBufferOut)
    memcpy(pcBufferOut, buffer + 1, lengthout - 1);
    
  // status byte removed
  if (pnLengthOut)
    *pnLengthOut = lengthout - 1;

  if (gcr_resp != NULL)
    *gcr_resp = rv;
    
  return SCARD_S_SUCCESS;
}

LONG GBP_Command(ReaderDesc * reader, DWORD nLengthIn, const BYTE pcBufferCmd[], DWORD * pnLengthOut, BYTE pcBufferOut[])
{
  LONG    rc;
  BYTE    rv;

  rc = GBP_CommandEx(reader, nLengthIn, pcBufferCmd, pnLengthOut, pcBufferOut, &rv);
  if (rc == SCARD_S_SUCCESS)
  {
    if (rv != GCORE_OK)
      rc = GBP_TranslateStatus(rv);
  } else
  {
    SCARD_Trace(TR_GCR_LOW, "Comm error %08lX", rc);
  }

  return rc;
}

static LONG GBP_TranslateStatus(BYTE gcr_status)
{
  switch (gcr_status)
  {
    case 0x00:
      return SCARD_S_SUCCESS;

    case 0x01:
      SCARD_Trace(TR_GCR_LOW, "Unknown driver or command");
      return SCARD_F_INTERNAL_ERROR;

    case 0x02:
      SCARD_Trace(TR_GCR_LOW, "Operation impossible with this driver");
      return SCARD_F_INTERNAL_ERROR;

    case 0x03:
      SCARD_Trace(TR_GCR_LOW, "Incorrect number of arguments");
      return SCARD_F_INTERNAL_ERROR;

    case 0x04:
      SCARD_Trace(TR_GCR_LOW, "Reader command unknown");
      return SCARD_F_INTERNAL_ERROR;
      
    case 0x05:
      SCARD_Trace(TR_GCR_LOW, "Response too long for the buffer");
      return SCARD_E_INSUFFICIENT_BUFFER;
      
    case 0x10:
      SCARD_Trace(TR_GCR_LOW, "ATR error : TS is not 3F/3B");
      return SCARD_E_INVALID_ATR;

    case 0x12:
      SCARD_Trace(TR_GCR_LOW, "Message too long");
      return SCARD_E_INSUFFICIENT_BUFFER;

    case 0x13:
      SCARD_Trace(TR_GCR_LOW, "Byte reading error");
      return SCARD_E_COMM_DATA_LOST;

    case 0x15:
      SCARD_Trace(TR_GCR_LOW, "Card powered down");
      return SCARD_W_UNPOWERED_CARD;

    case 0x1B:
      SCARD_Trace(TR_GCR_LOW, "A command has been sent with an incorrect number of parameters");
      return SCARD_F_INTERNAL_ERROR;

    case 0x1D:
      SCARD_Trace(TR_GCR_LOW, "ATR error : bad TCK");
      return SCARD_E_INVALID_ATR;

    case 0xA0:
      SCARD_Trace(TR_GCR_LOW, "ATR warning : card not supported");
      return SCARD_W_UNSUPPORTED_CARD;
    
    case 0xA1:
      SCARD_Trace(TR_GCR_LOW, "Card protocol error T=0/T=1");
      return SCARD_E_PROTO_MISMATCH;
      
    case 0xA2:
      SCARD_Trace(TR_GCR_LOW, "Card malfunction : no ATR or exchange timeout");
      return SCARD_W_UNRESPONSIVE_CARD;

    case 0xA3:
      SCARD_Trace(TR_GCR_LOW, "Persistent parity error");
      return SCARD_E_COMM_DATA_LOST;
      
    case 0xA4:
      SCARD_Trace(TR_GCR_LOW, "Card has aborted T=1 chaining");
      return SCARD_E_CARD_UNSUPPORTED;

    case 0xA5:
      SCARD_Trace(TR_GCR_LOW, "Reader has aborted T=1 chaining");
      return SCARD_E_CARD_UNSUPPORTED;
      
    case 0xA6:
      SCARD_Trace(TR_GCR_LOW, "Resync OK");
      return SCARD_S_SUCCESS;

    case 0xA7:
      SCARD_Trace(TR_GCR_LOW, "PPS error");
      return SCARD_W_UNSUPPORTED_CARD;

    case 0xA8:
      SCARD_Trace(TR_GCR_LOW, "Card already powered on");
      return SCARD_S_SUCCESS;

    case 0xB0:
      SCARD_Trace(TR_GCR_LOW, "PC-link command not supported");
      return SCARD_E_UNSUPPORTED_FEATURE;
      
    case 0xE4:
      SCARD_Trace(TR_GCR_LOW, "Card has returned an invalid procedure byte");
      return SCARD_E_CARD_UNSUPPORTED;

    case 0xE5:
      SCARD_Trace(TR_GCR_LOW, "Card interrupted the exchange after SW1");   
      return SCARD_S_SUCCESS;   /* Card stopped after SW1 */

    case 0xE7:
      SCARD_Trace(TR_GCR_LOW, "Error returned by the card (SW is not 9000)");
      return SCARD_S_SUCCESS;   /* Card didn't return 9000 */

    case 0xF7:
      SCARD_Trace(TR_GCR_LOW, "Card removed");
      return SCARD_W_REMOVED_CARD;

    case 0xF8:
      SCARD_Trace(TR_GCR_LOW, "Short circuit or card consuming too much");
      return SCARD_E_CARD_UNSUPPORTED;
      
    case 0xFB:
      SCARD_Trace(TR_GCR_LOW, "Card missing");
      return SCARD_E_NO_SMARTCARD;

  }

  SCARD_Trace(TR_GCR_LOW, "Unknown GCR error (%02X)", gcr_status);
  return ((SCARD_F_INTERNAL_ERROR - 1) | 0x1000 | gcr_status);
}


