/*
  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.

  Copyright (c) 2000-2008 SpringCard - www.springcard.com
  See COPYING for copyright and legal info

  pssc_high.c
  -----------
  High level exported functions
  

  JDA 12/06/07 : rewriting - all GemCore related stuff moved to pcsc_low.c

*/

#include "springcard_i.h"

/****************************************************************************
 *                                                                          *
 * Global data                                                              *
 *                                                                          *
 ****************************************************************************/

#ifdef __palmos__
static SCARD_CONTEXT_ST CONTEXT;
static SCARD_HANDLE_ST CARDS[SLOT_COUNT];
#endif

#define return_err(c,x) { if (c != NULL) { c->last_error = x; } return x; }

SPRINGCARD_LIB void SPRINGCARD_API SCardMessageBox(LPCTSTR szMessage);

SPRINGCARD_LIB SCARD_IO_REQUEST g_rgSCardT0Pci = { SCARD_PROTOCOL_T0, sizeof(SCARD_IO_REQUEST) };
SPRINGCARD_LIB SCARD_IO_REQUEST g_rgSCardT1Pci = { SCARD_PROTOCOL_T1, sizeof(SCARD_IO_REQUEST) };
SPRINGCARD_LIB SCARD_IO_REQUEST g_rgSCardRawPci = { SCARD_PROTOCOL_RAW, sizeof(SCARD_IO_REQUEST) };

#ifdef WIN32
#define USE_REAL_MUTEX
#endif

#ifdef USE_REAL_MUTEX

static HANDLE springcard_mtx = INVALID_HANDLE_VALUE;

BOOL create_mutex(SCARD_CONTEXT_ST * context)
{
  if (springcard_mtx != INVALID_HANDLE_VALUE)
    return TRUE;                /* Already instancied */
  springcard_mtx = CreateMutex(NULL, FALSE, _T("SpringCard_Mtx"));
  if (springcard_mtx == INVALID_HANDLE_VALUE)
    return FALSE;
  return TRUE;
}

void destroy_mutex(SCARD_CONTEXT_ST * context)
{
  SCARD_Trace(TR_COMM_LOW, "destroy_mutex:Handle=%08lX", springcard_mtx);
  
  if (springcard_mtx != INVALID_HANDLE_VALUE)
    CloseHandle(springcard_mtx);
  springcard_mtx = INVALID_HANDLE_VALUE;
}

BOOL acquire_mutex(SCARD_CONTEXT_ST * context)
{
  DWORD   dwRetcode;
  if (springcard_mtx == INVALID_HANDLE_VALUE)
  {
    if (context != NULL)
      context->last_error = SCARD_E_NOT_READY;
    return FALSE;               /* Not created */
  }
  dwRetcode = WaitForSingleObject(springcard_mtx, 1000);
  if (dwRetcode == WAIT_TIMEOUT)
  {
    if (context != NULL)
      context->last_error = SCARD_E_SHARING_VIOLATION;
    return FALSE;               /* Still in use after 1s */
  }
  return TRUE;
}

void release_mutex(SCARD_CONTEXT_ST * context)
{
  if (springcard_mtx == INVALID_HANDLE_VALUE)
    return;
  ReleaseMutex(springcard_mtx);
}

#else

BOOL create_mutex(SCARD_CONTEXT_ST * context)
{
  return TRUE;
}

void destroy_mutex(SCARD_CONTEXT_ST * context)
{
  //
}
BOOL acquire_mutex(SCARD_CONTEXT_ST * context)
{
  return TRUE;
}

void release_mutex(SCARD_CONTEXT_ST * context)
{
  //
}
#endif

/****************************************************************************
 *                                                                          *
 * PC/SC function implementation - conforms to winscard.h from MS           *
 *                                                                          *
 ****************************************************************************/

/*
 * Create a context - only one instance is allowed !!!
 */
SPRINGCARD_LIB LONG SPRINGCARD_API SCardEstablishContext(IN DWORD dwScope, IN LPCVOID pvReserved1, IN LPCVOID pvReserved2, OUT SCARD_CONTEXT_ST **phContext)
{
  int     i;
  LONG    rc;
  SCARD_CONTEXT_ST *context;

  SCARD_Trace(TR_PCSC_HIGH, "SCardEstablishContext");

  /* Check parameters */
  if (phContext == NULL)
    return SCARD_E_INVALID_PARAMETER;

  /* Context allocation */
#ifndef __palmos__
  context = (SCARD_CONTEXT_ST *) malloc(sizeof(SCARD_CONTEXT_ST));
#else
  context = &CONTEXT;
#endif
  /* MUST be done there... overwhise, in some cases when called from VB we lose the pointer */
  *phContext = (SCARDCONTEXT) context;

  /* Can't be instancied more than once */
#ifdef WIN32
  context->mutex = INVALID_HANDLE_VALUE;
#else
  context->mutex = FALSE;
#endif
  if (!create_mutex(context))
    return context->last_error;

  /* Set default context values */
  context->enabled = FALSE;
  context->start_count = 0;
  context->slot_count = SLOT_COUNT;
  for (i = 0; i < SLOT_COUNT; i++)
    context->cards[i] = NULL;

  context->desc.bSeq = 0;

  /* Startup reader */
  rc = RDR_Open(context);
  if (rc != SCARD_S_SUCCESS)
  {
    destroy_mutex(context);
#ifndef __palmos__
    free(context);
#endif
    return SCARD_E_NO_READERS_AVAILABLE;
  }

  /* Stop reader now for PDAs */
  RDR_Suspend(context);

  /* OK ! */
  release_mutex(context);
  return SCARD_S_SUCCESS;
}

/*
 * Release the context instance
 */
SPRINGCARD_LIB LONG SPRINGCARD_API SCardReleaseContext(IN SCARD_CONTEXT_ST *hContext)
{
#ifndef __palmos__
  int     i;
#endif
  SCARD_CONTEXT_ST *context = (SCARD_CONTEXT_ST *) hContext;

  SCARD_Trace(TR_PCSC_HIGH, "SCardReleaseContext");

  if (context == NULL)
    return SCARD_E_INVALID_HANDLE;
  if (!acquire_mutex(context))
    return context->last_error;

  /* Stop reader */
  RDR_Close(context);

  /* Kill context handle */
  destroy_mutex(context);
#ifndef __palmos__
  for (i = 0; i < SLOT_COUNT; i++)
    if (context->cards[i] != NULL)
      free(context->cards[i]);
  free(context);
#endif

  /* Done */
  return SCARD_S_SUCCESS;
}

/*
 * Enum available readers
 */
SPRINGCARD_LIB LONG SPRINGCARD_API SCardListReaders(IN SCARD_CONTEXT_ST *hContext, IN LPCTSTR mszGroups, OUT LPTSTR mszReaders, IN OUT LPDWORD pcchReaders)
{
  SCARD_CONTEXT_ST *context = (SCARD_CONTEXT_ST *) hContext;
  DWORD   len;
  DWORD   i;
  TCHAR  *p;

  if (context == NULL)
    return SCARD_E_INVALID_HANDLE;
  if ((pcchReaders == NULL) && (mszReaders == NULL))
    return_err(context, SCARD_E_INVALID_PARAMETER);

  /* Get size */
  {
    char   *p;
    p = context->names;
    len = (DWORD) strlen(p) + 1;
    p += strlen(p) + 1;
    len += (DWORD) (strlen(p) + 2);
  }

  /* What to do ? */
  if ((mszReaders == NULL) && (pcchReaders != NULL))
  {
    /* Get the size */
    *pcchReaders = len;
    return SCARD_S_SUCCESS;
  }

  if (mszReaders != NULL)
  {
    /* Keep the pointer */
    p = mszReaders;
    /* Check supplied length */
    if (pcchReaders != NULL)
    {
#ifndef __palmos__
      if (*pcchReaders == 0)
      {
        /* I must allocate the buffer */
        p = (TCHAR *) malloc(len * sizeof(TCHAR));
        if (p == NULL)
          return_err(context, SCARD_E_NO_MEMORY);

        *((TCHAR **) mszReaders) = p;
      } else
#endif
      if (*pcchReaders < len)
      {
        /* Buffer is too short */
        len = *pcchReaders;
      }
    }
    /* Buffer is ready, copy the data */
    for (i = 0; i < len; i++)
      p[i] = context->names[i];

    *pcchReaders = len;
  }

  /* OK ! */
  return SCARD_S_SUCCESS;
}


/*
 * Connect to a card on the selected reader
 */
SPRINGCARD_LIB LONG SPRINGCARD_API SCardConnect(IN SCARD_CONTEXT_ST *hContext,
                                                IN LPCTSTR szReader,
                                                IN DWORD dwShareMode,
                                                IN DWORD dwPreferredProtocols,
                                                OUT SCARD_HANDLE_ST **phCard,
                                                OUT LPDWORD pdwActiveProtocol)
{
  SCARD_CONTEXT_ST *context = (SCARD_CONTEXT_ST *) hContext;
  SCARD_HANDLE_ST *card;
  LONG    rc;
  BYTE    slot;
  int     i;
  const char *p;

  if (context == NULL)
    return SCARD_E_INVALID_HANDLE;
  if ((szReader == NULL) || (phCard == NULL))
    return_err(context, SCARD_E_INVALID_PARAMETER);

  if (!acquire_mutex(context))
    return context->last_error;

  if (_tcscmp(_T("A"), szReader) == 0)
  {
    slot = 0;
  } else if (_tcscmp(_T("B"), szReader) == 0)
  {
    slot = 1;
  } else
  {
    /* Find reader */
    p = context->names;
    slot = 0;
    for (;;)
    {
      for (i = 0; i < (int) strlen(p); i++)
        if (p[i] != szReader[i])
          break;
      if (i >= (int) strlen(p))
        break;

      slot++;
      p += strlen(p) + 1;
      if ((*p == '\0') || (slot >= SLOT_COUNT))
      {
        release_mutex(context);
        return_err(context, SCARD_E_UNKNOWN_READER);
      }
    }
  }

  /* Check if slot is correct */
  if (slot > context->slot_count)
    return_err(context, SCARD_E_INVALID_PARAMETER);

  /* Check if slot is available */
  if (context->cards[slot] != NULL)
  {
    release_mutex(context);
    return_err(context, SCARD_E_SHARING_VIOLATION);
  }

  /* Startup reader if required */
  rc = RDR_Resume(context);

  if (rc != SCARD_S_SUCCESS)
  {
    release_mutex(context);
    return_err(context, rc);
  }

  /* Allocate card */
#ifndef __palmos__
  card = (SCARD_HANDLE_ST *) malloc(sizeof(SCARD_HANDLE_ST));
  if (card == NULL)
  {
    release_mutex(context);
    return_err(context, SCARD_E_NO_MEMORY);
  }
#else
  card = &CARDS[slot];
#endif

  context->cards[slot] = card;
  card->context = context;
  card->atr_len = 0;
  card->run_protocol = 0;
  card->status = SCARD_ABSENT;
  card->slot = slot;
  card->timeout = CARD_TIMEOUT_DEFAULT;

  SCARD_Trace(TR_PCSC_HIGH, "Timeout set to %ld", CARD_TIMEOUT_DEFAULT);

  /* Check if there is a card in the reader */
  rc = SCARD_Status(card);

  if (rc != SCARD_S_SUCCESS)
  {
    RDR_Suspend(context);
#ifndef __palmos__
    free(card);
#endif
    context->cards[slot] = NULL;
    SCARD_Trace(TR_PCSC_HIGH, "GetStatus 1 failed");
    release_mutex(context);
    return_err(context, rc);
  }

  if (card->status == SCARD_ABSENT)
  {
    RDR_Suspend(context);
#ifndef __palmos__
    free(card);
#endif
    context->cards[slot] = NULL;
    release_mutex(context);
    return_err(context, SCARD_E_NO_SMARTCARD);
  }

  /* Try to establish dialog with the card */
  card->ask_protocol = dwPreferredProtocols % 0x00010000;

  if (card->ask_protocol & SCARD_PROTOCOL_RAW)
  {
    /* Memory cards */
    /* ------------ */
    
    rc = SCARD_PowerUp_Raw(card);

  } else
  if (card->ask_protocol & SCARD_PROTOCOL_Tx)
  {
    /* T=0 or T=1 cards */
    /* ---------------- */
    
    rc = SCARD_PowerUp(card);

  } else
    rc = SCARD_E_INVALID_VALUE;

  if (rc != SCARD_S_SUCCESS)
  {
    /* Power-up failed */
    RDR_Suspend(context);
#ifndef __palmos__
    free(card);
#endif
    context->cards[slot] = NULL;
    SCARD_Trace(TR_PCSC_HIGH, "PowerUp failed");
    release_mutex(context);
    return_err(context, rc);
  }

  rc = SCARD_Status(card);
  if (rc != SCARD_S_SUCCESS)
  {
    /* Get status failed */
    RDR_Suspend(context);
#ifndef __palmos__
    free(card);
#endif
    context->cards[slot] = NULL;
    SCARD_Trace(TR_PCSC_HIGH, "GetStatus 2 failed");
    release_mutex(context);
    return_err(context, rc);
  }

  if (card->status < SCARD_POWERED)
  {
    RDR_Suspend(context);
#ifndef __palmos__
    free(card);
#endif
    context->cards[slot] = NULL;
    SCARD_Trace(TR_PCSC_HIGH, "Card not powered");
    release_mutex(context);
    return SCARD_E_CARD_UNSUPPORTED;
  }

  if (pdwActiveProtocol != NULL)
    *pdwActiveProtocol = card->run_protocol;

  /* OK ! */
  *phCard = (SCARDHANDLE) card;
  release_mutex(context);

  SCARD_Trace(TR_PCSC_HIGH, "SCardConnect OK");

  return SCARD_S_SUCCESS;
}

/*
 * Reconnect from a card
 */
SPRINGCARD_LIB LONG SPRINGCARD_API SCardReconnect(IN SCARDHANDLE hCard,
                                                  IN DWORD dwShareMode,
                                                  IN DWORD dwPreferredProtocols,
                                                  IN DWORD dwInitialization,
                                                  OUT LPDWORD pdwActiveProtocol)
{
  SCARD_CONTEXT_ST *context;
  SCARD_HANDLE_ST *card = (SCARD_HANDLE_ST *) hCard;
  LONG    rc;

  if (card == NULL)
    return SCARD_E_INVALID_HANDLE;
  context = card->context;
  if (context == NULL)
    return SCARD_E_INVALID_HANDLE;

  if (!acquire_mutex(context))
    return context->last_error;

  /* Startup reader if required */
  rc = RDR_Resume(context);

  if (rc != SCARD_S_SUCCESS)
  {
    release_mutex(context);
    return_err(context, rc);
  }

  /* Check if there is a card in the reader */
  rc = SCARD_Status(card);

  if (rc != SCARD_S_SUCCESS)
  {
    RDR_Suspend(context);
    SCARD_Trace(TR_PCSC_HIGH, "GetStatus 1 failed");
    release_mutex(context);
    return_err(context, rc);
  }

  if (card->status == SCARD_ABSENT)
  {
    RDR_Suspend(context);
    release_mutex(context);
    return_err(context, SCARD_E_NO_SMARTCARD);
  }

  /* What to do with the card ? */
  if (dwInitialization == SCARD_UNPOWER_CARD)
  {
    SCARD_PowerDown(card);
  }

  if ((dwInitialization == SCARD_UNPOWER_CARD) || (dwInitialization == SCARD_RESET_CARD))
  {
    /* Try to establish dialog with the card */
    card->ask_protocol = dwPreferredProtocols % 0x00010000;

    if (card->ask_protocol & SCARD_PROTOCOL_RAW)
    {
      /* Memory cards */
      /* ------------ */
      
      rc = SCARD_PowerUp_Raw(card);

  } else
  if (card->ask_protocol & SCARD_PROTOCOL_Tx)
    {
      /* T=0 or T=1 cards */
      /* ---------------- */

      rc = SCARD_PowerUp(card);

    } else
      rc = SCARD_E_INVALID_VALUE;

    if (rc != SCARD_S_SUCCESS)
    {
      /* Power-up failed */
      RDR_Suspend(context);
      SCARD_Trace(TR_PCSC_HIGH, "PowerUp failed");
      release_mutex(context);
      return_err(context, rc);
    }
  }

  rc = SCARD_Status(card);
  if (rc != SCARD_S_SUCCESS)
  {
    /* Get status failed */
    RDR_Suspend(context);
    SCARD_Trace(TR_PCSC_HIGH, "GetStatus 2 failed");
    release_mutex(context);
    return_err(context, rc);
  }

  if (card->status < SCARD_POWERED)
  {
    RDR_Suspend(context);
    SCARD_Trace(TR_PCSC_HIGH, "Card not powered");
    release_mutex(context);
    return SCARD_E_CARD_UNSUPPORTED;
  }

  if (pdwActiveProtocol != NULL)
    *pdwActiveProtocol = card->run_protocol;

  /* OK ! */
  release_mutex(context);

  SCARD_Trace(TR_PCSC_HIGH, "SCardReconnect OK");

  return SCARD_S_SUCCESS;
}

/*
 * Disconnect from a card
 */
SPRINGCARD_LIB LONG SPRINGCARD_API SCardDisconnect(IN SCARDHANDLE hCard, IN DWORD dwDisposition)
{
  SCARD_CONTEXT_ST *context;
  SCARD_HANDLE_ST *card = (SCARD_HANDLE_ST *) hCard;
  int     i;
  BOOL    still_active;

  SCARD_Trace(TR_PCSC_HIGH, "SCardDisconnect");

  if (card == NULL)
    return SCARD_E_INVALID_HANDLE;
  context = card->context;
  if (context == NULL)
    return SCARD_E_INVALID_HANDLE;
  if (!acquire_mutex(context))
    return context->last_error;

  if (dwDisposition >= SCARD_EJECT_CARD)
  {
    /* Power-off the card */
    SCARD_PowerDown(card);
    still_active = FALSE;
  } else if (dwDisposition >= SCARD_UNPOWER_CARD)
  {
    /* Power-off the card */
    SCARD_PowerDown(card);
    still_active = TRUE;
  } else if (dwDisposition == SCARD_RESET_CARD)
  {
    /* Reset the card but let it powered */
    SCARD_PowerUp(card);
    still_active = TRUE;
  } else
  {
    /* Do nothing with the card, let it powered */
    still_active = TRUE;
  }

  /* Kill card handle and check if others are active */
  for (i = 0; i < SLOT_COUNT; i++)
  {
    if (context->cards[i] == NULL)
      continue;
    if (context->cards[i] == card)
    {
      context->cards[i] = NULL;
    } else
    {
      still_active = TRUE;
    }
  }
#ifndef __palmos__
  free(card);
#endif

  /* If I was the last card in use, put the reader in idle mode */
  if (!still_active)
  {
    RDR_Suspend(context);
  }

  /* OK ! */
  release_mutex(context);
  return SCARD_S_SUCCESS;
}

/*
 * Retrieve card status
 */
SPRINGCARD_LIB LONG SPRINGCARD_API SCardStatus(IN SCARDHANDLE hCard,
                                               OUT LPTSTR mszReaderNames,
                                               IN OUT LPDWORD pcchReaderLen,
                                               OUT LPDWORD pdwState,
                                               OUT LPDWORD pdwProtocol,
                                               OUT LPBYTE pbAtr,
                                               OUT LPDWORD pbcAtrLen)
{
  LONG rc;
  
  SCARD_CONTEXT_ST *context;
  SCARD_HANDLE_ST *card = (SCARD_HANDLE_ST *) hCard;

  SCARD_Trace(TR_PCSC_HIGH, "SCardStatus");

  /* Check parameters */
  if (card == NULL)
    return SCARD_E_INVALID_HANDLE;
  context = card->context;
  if (context == NULL)
    return SCARD_E_INVALID_HANDLE;
  if (!acquire_mutex(context))
    return context->last_error;

  /* Update the status */
  rc = SCARD_Status(card);
  
  /* Translate (non)error codes to OK */
  switch (rc)
  {
  	case SCARD_E_NO_SMARTCARD      :
  	case SCARD_E_UNKNOWN_CARD      :
  	case SCARD_E_PROTO_MISMATCH    :
  	case SCARD_E_INVALID_ATR       :
  	case SCARD_E_CARD_UNSUPPORTED  :
  	case SCARD_W_UNSUPPORTED_CARD  :
  	case SCARD_W_UNRESPONSIVE_CARD :
  	case SCARD_W_UNPOWERED_CARD    :
  	case SCARD_W_RESET_CARD        :
  	case SCARD_W_REMOVED_CARD      : rc = SCARD_S_SUCCESS;
  	                                 break;
  }

  /* Get values */
  if (pdwState != NULL)
    *pdwState = card->status;
  if (pdwProtocol != NULL)
    *pdwProtocol = card->run_protocol;
  if (pbAtr != NULL)
    memcpy(pbAtr, card->atr, 32);
  if (pbcAtrLen != NULL)
    *pbcAtrLen = card->atr_len;

  release_mutex(context);

  SCARD_Trace(TR_PCSC_HIGH, "SCardStatus (%08lX)", rc);
  return rc;
}

/*
 * Direct card control
 * -------------------
 */
SPRINGCARD_LIB LONG SPRINGCARD_API SCardControl(IN SCARD_HANDLE_T hCard,
                                                IN LPBYTE pbSendBuffer,
                                                IN DWORD cbSendLength,
                                                IN LPBYTE pbRecvBuffer,
                                                IN OUT LPDWORD pcbRecvLength)
{
  LONG    rc;

  SCARD_CONTEXT_ST *context;
  SCARD_HANDLE_ST *card = (SCARD_HANDLE_ST *) hCard;

  /* Check parameters */
  if (card == NULL)
    return SCARD_E_INVALID_HANDLE;
  context = card->context;
  if (context == NULL)
    return SCARD_E_INVALID_HANDLE;

  /* Pass the buffer directly to the GemCore */
  if (!acquire_mutex(context))
    return context->last_error;

  rc = SCARD_Control(card, pbSendBuffer, cbSendLength, pbRecvBuffer, *pcbRecvLength, pcbRecvLength);

  release_mutex(context);

  if (rc != SCARD_S_SUCCESS)
    return_err(context, rc);

  return SCARD_S_SUCCESS;
}

/*
 * Exchange APDU with the card
 */
SPRINGCARD_LIB LONG SPRINGCARD_API SCardTransmit(IN SCARDHANDLE hCard,
                                                 IN LPCSCARD_IO_REQUEST pioSendPci,
                                                 IN LPBYTE pbSendBuffer,
                                                 IN DWORD cbSendLength,
                                                 IN OUT LPSCARD_IO_REQUEST pioRecvPci,
                                                 OUT LPBYTE pbRecvBuffer,
                                                 IN OUT LPDWORD pcbRecvLength)
{
  LONG    rc;
  DWORD   cbRecvLength;
  SCARD_HANDLE_ST *card = (SCARD_HANDLE_ST *) hCard;
  SCARD_CONTEXT_ST *context;

  /* Check parameters */
  if (card == NULL)
    return SCARD_E_INVALID_HANDLE;
  context = (SCARD_CONTEXT_ST *) card->context;
  if (context == NULL)
    return SCARD_E_INVALID_HANDLE;

  if (pbSendBuffer == NULL)
    return_err(context, SCARD_E_INVALID_PARAMETER);
  if (pbRecvBuffer == NULL)
    return_err(context, SCARD_E_INVALID_PARAMETER);
  if (pcbRecvLength == NULL)
    return_err(context, SCARD_E_INVALID_PARAMETER);

  if (cbSendLength < APDU_SIZE_MIN)
    return_err(context, SCARD_E_INVALID_VALUE);
  if (cbSendLength > APDU_SIZE_MAX)
    return_err(context, SCARD_E_INVALID_VALUE);
  if (*pcbRecvLength > RESP_SIZE_MAX)
    *pcbRecvLength = RESP_SIZE_MAX;

  /* Non-reentrant */
  if (!acquire_mutex(context))
    return context->last_error;

  /* Main code */
  cbRecvLength = *pcbRecvLength;

  rc = SCARD_Transmit(card, pbSendBuffer, cbSendLength, pbRecvBuffer, cbRecvLength, pcbRecvLength);

  release_mutex(context);

  if (rc != SCARD_S_SUCCESS)
    SCARD_Trace(TR_PCSC_HIGH, "SCardTransmit -> %08lX", rc);

  /* Done */
  return rc;
}

/****************************************************************************
 *                                                                          *
 * more PC/SC function implementation - conforms to winscard.h from MUSCLE  *
 *                                                                          *
 ****************************************************************************/

/*
 * Define timeout during APDU Exchanges
 */
SPRINGCARD_LIB LONG SPRINGCARD_API SCardSetTimeout(IN SCARDHANDLE hCard, IN DWORD dwTimeout)
{
  SCARD_HANDLE_ST *card = (SCARD_HANDLE_ST *) hCard;

  /* Check parameters */
  if (card == NULL)
    return SCARD_E_INVALID_HANDLE;
  if (dwTimeout > CARD_TIMEOUT_MAX)
    return SCARD_E_INVALID_PARAMETER;

  card->timeout = dwTimeout;
  SCARD_Trace(TR_PCSC_HIGH, "Timeout set to %ld", dwTimeout);

  return SCARD_S_SUCCESS;
}

/****************************************************************************
 *                                                                          *
 * even more PC/SC function implementation - SpringCard specific            *
 *                                                                          *
 ****************************************************************************/

SPRINGCARD_LIB LONG SPRINGCARD_API SCardGetLibrary(OUT TCHAR *library, IN WORD len)
{
  TCHAR buffer[MAX_PATH];
  _stprintf(buffer, _T("SPRINGCARD SPRINGCARD API %X.%02X [%d]"), VERSION_MAJOR, VERSION_MINOR, VERSION_BUILD);

  if (library == NULL)
    return SCARD_E_INVALID_PARAMETER;
  if (len == 0)
    return SCARD_E_INVALID_PARAMETER;

  _tcsncpy(library, buffer, len);
  library[len - 1] = '\0';

  return SCARD_S_SUCCESS;
}

/****************************************************************************
 *                                                                          *
 * "Helpers" function - not in winscard.h, but very usefull for VB users    *
 *                                                                          *
 ****************************************************************************/

/*
 * Allocate a buffer
 */
SPRINGCARD_LIB LONG SPRINGCARD_API SCardAllocMemory(IN SCARD_CONTEXT_ST *hContext, IN OUT LPVOID * pvMem, IN DWORD dwSize)
{
  SCARD_CONTEXT_ST *context = (SCARD_CONTEXT_ST *) hContext;
  void   *p;

  p = (void *) malloc(dwSize);
  if (p == NULL)
    return_err(context, SCARD_E_NO_MEMORY);

  *pvMem = p;

  return SCARD_S_SUCCESS;
}

/*
 * Free memory allocated here
 */
SPRINGCARD_LIB LONG SPRINGCARD_API SCardFreeMemory(IN SCARD_CONTEXT_ST *hContext, IN LPVOID pvMem)
{
#ifndef __palmos__
  free(pvMem);
#endif
  return SCARD_S_SUCCESS;
}

/*
 * Translate a string into binary
 */
SPRINGCARD_LIB LONG SPRINGCARD_API SCardStringToBin(IN SCARD_CONTEXT_ST *hContext, IN OUT LPVOID pvResult, IN DWORD dwResultLen, IN LPCTSTR szText)
{
  SCARD_CONTEXT_ST *context = (SCARD_CONTEXT_ST *) hContext;
  DWORD   i, j, l;
  BYTE   *p;

  if (pvResult == NULL)
    return_err(context, SCARD_E_INVALID_PARAMETER);
  if (szText == NULL)
    return_err(context, SCARD_E_INVALID_PARAMETER);

  /* Decode */
  p = (BYTE *) pvResult;
  i = 0;
  j = 0;
  l = (DWORD) _tcsclen(szText);
//  if (l > TLP_DATA_BUF_SIZE) l = TLP_DATA_BUF_SIZE;

  while (i < l)
  {
    BYTE    b;
#ifndef __palmos__
    TCHAR   q;
    char    tmp[3];
#endif

    if (j >= dwResultLen)
      return_err(context, SCARD_E_INSUFFICIENT_BUFFER);

#ifndef __palmos__
    q = szText[i++];
    if (q == '\0')
      break;
    tmp[0] = (char) q;
    q = szText[i++];
    if (q == '\0')
      break;
    tmp[1] = (char) q;
    tmp[2] = '\0';
    b = c2bin(tmp, 2);
#else
    b = c2bin((char *) &szText[i], 2);
    i += 2;
#endif

    p[j++] = b;
  }

  /* OK ! */
  return SCARD_S_SUCCESS;
}

/*
 * Translate a buffer encoded in HEX/ASCII into a numerical BYTE value
 */
SPRINGCARD_LIB LONG SPRINGCARD_API SCardHexToByte(IN SCARD_CONTEXT_ST *hContext, IN LPBYTE pbResult, IN LPCTSTR szText)
{
  SCARD_CONTEXT_ST *context = (SCARD_CONTEXT_ST *) hContext;

  if (pbResult == NULL)
    return_err(context, SCARD_E_INVALID_PARAMETER);
  if (szText == NULL)
    return_err(context, SCARD_E_INVALID_PARAMETER);

#ifndef __palmos__
  {
    char    tmp[3];
    tmp[0] = (char) szText[0];
    tmp[1] = (char) szText[1];
    tmp[2] = '\0';
    *pbResult = c2bin(tmp, 2);
  }
#else
  *pbResult = c2bin((char *) szText, 2);
#endif

  /* OK ! */
  return SCARD_S_SUCCESS;
}

/*
 * Translate a buffer encoded in HEX/ASCII into a string
 */
SPRINGCARD_LIB LONG SPRINGCARD_API SCardHexToString(IN SCARD_CONTEXT_ST *hContext, IN LPTSTR szResult, IN DWORD dwResultLen, IN LPCTSTR szText)
{
  DWORD   i, j, l;
  TCHAR  *p;
  SCARD_CONTEXT_ST *context = (SCARD_CONTEXT_ST *) hContext;

  if (szResult == NULL)
    return_err(context, SCARD_E_INVALID_PARAMETER);
  if (szText == NULL)
    return_err(context, SCARD_E_INVALID_PARAMETER);

  /* Decode */
  p = szResult;
  i = 0;
  j = 0;
  l = (DWORD) _tcsclen(szText);
//  if (l > TLP_DATA_BUF_SIZE) l = TLP_DATA_BUF_SIZE;

  while (i < l)
  {
    BYTE    b;
#ifndef __palmos__
    TCHAR   q;
    char    tmp[3];

    if (j >= dwResultLen)
      return_err(context, SCARD_E_INSUFFICIENT_BUFFER);

    q = szText[i++];
    if (q == '\0')
      break;
    tmp[0] = (char) q;
    q = szText[i++];
    if (q == '\0')
      break;
    tmp[1] = (char) q;
    tmp[2] = '\0';
    b = c2bin(tmp, 2);
    _stprintf(&p[j++], _T("%c"), b);
#else
    b = c2bin((char *) &szText[i], 2);
    i += 2;
    p[j++] = (char) b;
#endif
  }

  /* OK ! */
  return SCARD_S_SUCCESS;
}

/*
 * Translate a buffer into a string
 */
SPRINGCARD_LIB LONG SPRINGCARD_API SCardBinToString(IN SCARD_CONTEXT_ST *hContext,
                                                    IN OUT LPTSTR szResult,
                                                    IN DWORD dwResultLen,
                                                    IN OUT LPVOID pvMem,
                                                    IN DWORD dwSize)
{
  DWORD   i;
  BYTE   *p;
  SCARD_CONTEXT_ST *context = (SCARD_CONTEXT_ST *) hContext;

  if (szResult == NULL)
    return_err(context, SCARD_E_INVALID_PARAMETER);
  if (pvMem == NULL)
    return_err(context, SCARD_E_INVALID_PARAMETER);

  /* Required strlen is dwSize * 2 + 1 */
  i = 2;
  i *= dwSize;
  i++;
  i *= sizeof(TCHAR);           /* For UNICODE */

  if (dwResultLen < i)
    return_err(context, SCARD_E_INSUFFICIENT_BUFFER);

  /* Clear string */
  szResult[0] = (TCHAR) '\0';

  /* And copy data */
  p = (BYTE *) pvMem;
  for (i = 0; i < dwSize; i++)
  {
#ifndef __palmos__
    TCHAR   tmp[3];
    _stprintf(tmp, _T("%02X"), p[i]);
    _tcscat(szResult, tmp);
#else
    bin2c(p[i], &szResult[2 * i], 2);
#endif
  }

  /* OK ! */
  return SCARD_S_SUCCESS;
}

/*
 * Get length of a string
 */
SPRINGCARD_LIB LONG SCardStringLength(IN SCARD_CONTEXT_ST *hContext, IN LPCTSTR szText)
{
  if (szText == NULL)
    return 0;
  return (LONG) _tcsclen(szText);
}

/*
 * Retrieve last errors
 */
SPRINGCARD_LIB LONG SPRINGCARD_API SCardGetLastError(IN SCARD_CONTEXT_ST *hContext)
{
  SCARD_CONTEXT_ST *context = (SCARD_CONTEXT_ST *) hContext;
  if (context == NULL)
    return SCARD_E_INVALID_HANDLE;
  return context->last_error;
}

SPRINGCARD_LIB LONG SPRINGCARD_API SCardGCRGetLastError(IN SCARD_CONTEXT_ST *hContext)
{
//  SCARD_CONTEXT_ST *context = (SCARD_CONTEXT_ST *) hContext;
//  if (context == NULL)
//    return (LONG) -1;
//  return GetRerrno();

	//MessageBox(0, _T("SCardGCRGetLastError"), _T("SCARD_E_UNSUPPORTED_FEATURE"), 0);    
  return SCARD_E_UNSUPPORTED_FEATURE;
}

/*
 * Translate errors to string
 */
SPRINGCARD_LIB LPTSTR SPRINGCARD_API SCardErrorToString(IN SCARD_CONTEXT_ST *hContext, LONG dwErrCode)
{
  return PCSC_ErrorStr(dwErrCode);
}

/*
 * Translate errors to string
 */
SPRINGCARD_LIB LPTSTR SPRINGCARD_API SCardISOErrorToString(IN SCARD_CONTEXT_ST *hContext, BYTE sw1, BYTE sw2)
{
  return ISO7816_ErrorStr(sw1, sw2);
}

SPRINGCARD_LIB LPTSTR SPRINGCARD_API SCardGCRErrorToString(IN SCARD_CONTEXT_ST *hContext, DWORD dwErrCode)
{
//  return GetRerrStr((int) dwErrCode); 
  static TCHAR no_err[] = _T("Unsupported feature");
  return no_err;
}

/*
 * Display a MessageBox
 */
SPRINGCARD_LIB void SPRINGCARD_API SCardMessageBox(LPCTSTR szMessage)
{
#ifdef WIN32
  MessageBox(0, szMessage, _T("SpringCard SpringCard"), MB_APPLMODAL + MB_TOPMOST);
#else
  printf("SpringCard SpringCard : %s\n", szMessage);
#endif
}

/*
 * Select a string in a multi-string buffer
 */
SPRINGCARD_LIB LPTSTR SPRINGCARD_API SCardMultiStringItem(IN SCARD_CONTEXT_ST *hContext, IN LPCTSTR szMulti, IN DWORD dwItem)
{
  DWORD   i;
  TCHAR  *p;

  if (szMulti == NULL)
    return NULL;

  p = (TCHAR *) szMulti;
  i = 0;
  for (;;)
  {
    if (i == dwItem)
      return p;
    p += _tcslen(p);
    p++;
    if (*p == '\0')
      return NULL;
    i++;
  }
  return NULL;
}

/*
 * Count strings in a multi-string buffer
 */
SPRINGCARD_LIB DWORD SPRINGCARD_API SCardMultiStringCount(IN SCARD_CONTEXT_ST *hContext, IN LPCTSTR szMulti)
{
  TCHAR  *p;
  DWORD   count;

  if (szMulti == NULL)
    return 0;

  p = (TCHAR *) szMulti;
  count = 0;
  for (;;)
  {
    if (*p == '\0')
      return count;
    count++;
    p += _tcslen(p);
    p++;
  }
  return 0;
}

/*
 * Transactions
 */
SPRINGCARD_LIB LONG SPRINGCARD_API SCardBeginTransaction(IN SCARD_HANDLE_ST *hCard)
{
  return SCARD_S_SUCCESS;
}

SPRINGCARD_LIB LONG SPRINGCARD_API SCardCancelTransaction(IN SCARD_HANDLE_ST *hCard)
{
  return SCARD_S_SUCCESS;
}

SPRINGCARD_LIB LONG SPRINGCARD_API SCardEndTransaction(IN SCARD_HANDLE_ST *hCard, IN DWORD dwDisposition)
{
  return SCARD_S_SUCCESS;
}

/* Return GemCore Family */ 
SPRINGCARD_LIB BYTE SPRINGCARD_API SCardGetGCRFamily(void)
{
  return gcr_family;
}

