/*****************************************************************************
* Copyright (c)  Gemplus Developpement 1992 - 1997
*
* Name        : READER.H
*
* Description :
*
  Author      : Jean-Luc GIRAUD
  Compiler    : Visual C++, C (Linux)
  Host        : Windows 9x,NT, Linux

* $Id: reader.h,v 1.7 2000/09/19 10:30:21 rousseau Exp $
* $Log: reader.h,v $
* Revision 1.7  2000/09/19 10:30:21  rousseau
* ISO-7816_ErrorStr now adds the SW in hexa
*
* Revision 1.6  2000/09/18 15:17:06  lvictor
* First modifications to support the recent changes made by L.Rousseau to the lib.
* (L.Gauteron + L.Victor)
*
* Revision 1.5  2000/09/15 16:13:51  rousseau
* moved files from common/ to src/
*
* Revision 1.4  2000/09/04 15:29:07  giraud
* Change prototypes for error code handling functions
*
* Revision 1.3  2000/09/04 08:19:04  rousseau
* code cleanup, changed the API and name for [Init|Close]Serial
*
*
******************************************************************************
*
*Warning     :
*
*Remarks     :
*
*****************************************************************************/

#ifndef _READER_H_
#define _READER_H_

#include "sctypes.h"

#ifdef __palmos__
#include <PalmOS.h>
#endif

#ifdef __linux__
#include <termios.h>
#define NORM_BAUDRATE B9600
#define HIGH_BAUDRATE B38400
#endif

#ifdef WIN32
#include <windows.h>
#define NORM_BAUDRATE CBR_9600
#define HIGH_BAUDRATE CBR_38400
#endif

#ifndef TYPE_HANDLE
#error Fatale, cible non reconnue
#endif

/* Reader error codes */
typedef enum RD_ERRORS
{
RD_NO_ERROR = 0,
RD_UNKNOWN_COMMAND,
RD_DRIVER_LIMITATION,
RD_WRONG_ARG_LENGTH,
RD_PROTOCOL_ERROR,
RD_WRONG_ATR,                   /* 5 */
RD_WRONG_RD_INS,
RD_MSG_TOO_LONG,
RD_READ_ERROR,
RD_CARD_OFF,
RD_PROG_VOLT_ERROR,             /* 10 */
RD_UNKNOWN_PROTOCOL,
RD_LENGTH_ERROR,
RD_WRONG_NB_OF_PARAM,
RD_UNKNOWN_CARD_PROTOCOL,
RD_CARD_MUTE,                   /* 15 */
RD_CARD_PARITY_ERROR,
RD_CARD_WRONG_PROCEDURE_BYTE,
RD_TRANSFERT_TO_CARD_INTERRUPTED,
RD_CARD_REMOVED,
RD_SHORT_CIRCUIT,               /* 20 */
RD_CARD_ABSENT,
  /* read on serial port failed */
RD_READ_FAILED,
  /* reader complained about a transmission error */
RD_TRANS_ERROR,
  /* write to serial port failed */
RD_WRITE_FAILED,
  /* Response form reader exceeded buffer length */
RD_RESP_TOO_LONG,               /* 25 */
RD_READER_MUTE,
RD_STATUS_TOO_LONG,
RD_USELESS_ERROR,
RD_CHAINING_ERROR,
RD_RESP_TRUNCATED,              /* 30 */
RD_UNKNOWN_ERROR,

RD_NUM_ERRORS
}
t_rd_error;

/* No need to look into the details of this structure, */
/* detailed information is only useful for the library */
typedef struct RD
{
TYPE_HANDLE fd;                 /* Device file descriptor */
char    bSeq;                   /* Sequence for GBP dialog */
#ifdef __linux__
struct termios oldtio;          /* Port initial state (restored on exit) */
#endif
#ifdef WIN32
DCB     oldtio;                 /* Port initial state (restored on exit) */
#endif
DWORD   cmd_timeout;
BOOL    fatal_error;
}
ReaderDesc;

#define ATR_BUF_SIZE 600
#define ISO7816_ERROR_TEXT_SIZE 250

/****************************************************************************/
/*                                                                          */
/*  Get the readerlib error message number                                  */
/*                                                                          */
/*  RETURN VALUE: the error number                                          */
/*                                                                          */
/****************************************************************************/
// t_rd_error GetRerrno(void);

TCHAR  *GetRerrStr(t_rd_error nErr);
TCHAR  *ISO7816_ErrorStr(BYTE sw1,
                         BYTE sw2);

int     SerialOpen(ReaderDesc * reader,
                   TCHAR * device);
int     InitReader(ReaderDesc * reader);
void    SerialClose(ReaderDesc * reader);

/*ISO length size (1 in general) */
//#define GCR_ISO_LENGTH_SIZE 1

/* ISO SW size */
#define GCR_ISO_SIZE_SW 2

/* An APDU is at least 4 bytes long */
#define APDU_SIZE_MIN 4

/* Maximum size for a SCardTransmit buffer (max=apdu+Lc+data+Le) */
#define APDU_SIZE_MAX (4+1+256+1)

/* Maximum size for a SCardTransmit response (max=data+sw) */
#define RESP_SIZE_MAX (256+2)

/* Offset of the length byte in an TPDU */
#define GCR_ISO_OFFSET_LENGTH 4

/* Offset of the data in a TPDU */
#define GCR_ISO_OFFSET_TPDU_DATA 5

/* Maximum size of Data for ISO_Output */
#define GCR_MAX_ISO_OUTPUT_SIZE 252

/* Maximum size of Data for ISO_Input */
#define GCR_MAX_ISO_INPUT_SIZE 248



/* Command buffer size (max=cmd+adpu+Lc+data+Le) */
#define GCR_SEND_BUF_SIZE (1+4+1+256+1)
/* Response buffer size (max=data+sw) */
#define GCR_RESP_BUF_SIZE (1+256+2)

// Size of the command field in a gemcore command
#define GC_SIZE_CMD 1
// Size of the reader status
#define GC_SIZE_STATUS 1

// *** should disapear
// Offset of the status byte in a reader answer (CMD level)
#define GC_STATUS_OFFSET 0

// Offset of the command byte in a GemCore command (CMD level)
#define GC_OFFSET_CMD 0
// Offset of the APDU in a GemCore command (CMD level)
#define GC_OFFSET_APDU 1
// Offset of the Lc byte in the APDU of a Gemcore command
#define GC_OFFSET_LENGTH (GC_OFFSET_APDU+ISO_OFFSET_LENGTH)
//Offset of the Data in a TPDU Gemcore command
#define GC_OFFSET_TPDU_CMD_DATA (GC_OFFSET_APDU+ISO_OFFSET_TPDU_DATA)
// Offset of the DATA in a GemCore response (CMD level)
#define GC_OFFSET_RESP_DATA 1
// Nb of bytes added to the APDU in a GemCore command (CMD level)
#define GC_FORMAT_SIZE 1

// STAT byte offset in a response to CARD STATUS command (CMD level)
#define GC_OFFSET_STAT_BYTE 0
//Bit mask to get the reader power status
#define GC_MASK_POWER 0x02
// Bit mask to detect ICC presence
#define GC_MASK_ICC_PRESENCE 0x04
// size if CARD STATUS response
#define GC_SIZE_CARD_STATUS 6

//Maximum size of Data for ISO_Output before pcLongDataADPU is required
#define GC_ISO_OUTPUT_MAX_DATA_LENGTH 252

//Maximum size of Data for ISO_Output before pcLongDataADPU is required
#define GC_ISO_INPUT_MAX_DATA_LENGTH 248

//Maximum length for APDU command (ie CMD+2xlength+DataIn)
#define GC_ISO_EXC_CMD_MAX_APDU_LENGTH 254

//Maximum length for APDU response (ie Data + SW)
#define GC_ISO_EXC_RESP_MAX_APDU_LENGTH 254

//Length of the reader response to a transmit command
#define GC_READER_TRANSMIT_RESP_LENGTH 1


#define GCR_ISO_OUT             0x13
#define GCR_ISO_OUT_AUX         0x1B

#define GCR_ISO_IN              0x14
#define GCR_ISO_IN_AUX          0x1C

#define GCR_APDU                0x15
#define GCR_APDU_AUX            0x1D


#endif
