/*
 *	Name : SpringCard_S10.c
 *	----------------------
 *	Source code for the SpringCard_S10 Dll
 *
 *	(c) Pro-Active S.A.S. - http://www.pro-active.fr
 *
 *	
 *	History :
 *		01/11/2006	DPI	Creation
 *
 *---------------------------------------------------------------------------*/

#include "springcard_s10_i.h"


/*
 *	Global variables
 *	----------------
 *---------------------------------------------------------------------------*/
BOOL springcard_on   = FALSE;
BOOL springcard_card = FALSE;

SCARDCONTEXT SCardContext = NULL;
SCARDHANDLE SCardHandle   = NULL;
LONG SCardLastError;
DWORD SCardProtocol;
TCHAR SCardReaders[250];
TCHAR SCardErrorStr[250];

BOOL mutex = FALSE;
/*---------------------------------------------------------------------------*/
BOOL CardDirect(BYTE send_len, BYTE *send_data, BYTE recv_len, BYTE *recv_data, BYTE *status);
BOOL CardTransmit(BYTE send_len, BYTE *send_data, BYTE recv_len, BYTE *recv_data, BYTE *status);;
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*   Read Functions : Data Area , Protection Area , Security Area            */
/*---------------------------------------------------------------------------*/
BOOL S10ReadDataArea(BYTE *dest ,BYTE addr, BYTE len);
BOOL S10ReadProtectionArea(BYTE *dest);
BOOL S10ReadSecurityArea(BYTE *dest);

/*---------------------------------------------------------------------------*/
/*   Write Functions : Data Area , Protection Area , Security Area           */
/*---------------------------------------------------------------------------*/
BOOL S10WriteDataArea(BYTE *source, BYTE addr, BYTE len);
BOOL S10WriteProtectionArea(BYTE *source, BYTE addr, BYTE len);
BOOL S10UpdateSecurityArea(BYTE *source, BYTE addr, BYTE len);

BOOL S10PresentCardSecretCode(BYTE code3 , BYTE code2, BYTE code1);



/*																STATUS : VALID
 *	Name : ReaderOpen
 *	-----------------
 *	Open the reader and retrieve its names (first name is the regular 
 *	smartcard slot, second name is the internal SIM/SAM slot).
 *---------------------------------------------------------------------------*/
BOOL ReaderOpen(void)
{
  DWORD l;

  if (!springcard_on)
  {
    /* Init PC/SC layer */
    SCardLastError = SCardEstablishContext(SCARD_SCOPE_SYSTEM, NULL, NULL, &SCardContext);
    if (SCardLastError != SCARD_S_SUCCESS)
      goto error;

    /* Get readers list */
    l = sizeof(SCardReaders);
    SCardLastError = SCardListReaders(SCardContext, NULL, SCardReaders, &l);
    if (SCardLastError != SCARD_S_SUCCESS)
      goto error;

    /* OK ! */
    /* ---- */
    springcard_on = TRUE;
  }

  /* Display firmware */
  return TRUE;
  
error:
  return FALSE;
}

/*																STATUS : VALID
 *	Name : ReaderClose
 *	------------------
 *	Close the reader
 *---------------------------------------------------------------------------*/
BOOL ReaderClose(void)
{
  if (springcard_on)
  {
    CardClose();

    /* Free PC/SC layer */
    SCardReleaseContext(SCardContext);

    springcard_on = FALSE;
    return TRUE;
  }
  
  return FALSE;
}

/*																STATUS : VALID
 *	Name : CardOpen
 *	---------------
 *	Get access to a smartcard
 *---------------------------------------------------------------------------*/
BOOL CardOpen(void)
{
  /* SpringCard must be opened */
  if (!springcard_on) 
    return FALSE;

  if (!springcard_card)
  {
    SCardLastError = SCardConnect(SCardContext,
                                  SCardMultiStringItem(SCardContext, SCardReaders, 0),
                                  SCARD_SHARE_DIRECT,
                                  SCARD_PROTOCOL_RAW | SCARD_PROTOCOL_S10,
                                  &SCardHandle,
                                  &SCardProtocol);
    if (SCardLastError != SCARD_S_SUCCESS) 
      return FALSE;
	
    /* OK ! */
    /* ---- */
    springcard_card = TRUE;
  }

  return TRUE;
}

/*																STATUS : VALID
 *	Name : CardClose
 *	----------------
 *	Close the smartcard
 *---------------------------------------------------------------------------*/
BOOL CardClose(void)
{
  if (springcard_card)
  {
    /* Try to eject the card */
	SCardDisconnect(SCardHandle, SCARD_UNPOWER_CARD);
	springcard_card = FALSE;
	return TRUE;
  }

  return FALSE;
}

/*																STATUS : VALID
 *	Name : CardDirect
 *	-----------------
 *	Raw exchange with the GemCore chipset
 *---------------------------------------------------------------------------*/
BOOL CardDirect(BYTE send_len, BYTE *send_data, BYTE recv_len, BYTE *recv_data, BYTE *status)
{
  if (!springcard_on)
	return FALSE;

  if (!springcard_card)
	return FALSE;

  SCardLastError = SCardS10CardDirect(SCardHandle, send_len, send_data, recv_len, recv_data, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return FALSE; 

  return TRUE; 
}
/*---------------------------------------------------------------------------*/
BOOL CardTransmit(BYTE send_len, BYTE *send_data, BYTE recv_len, BYTE *recv_data, BYTE *status)
{
  if (!springcard_on)
	return FALSE;

  if (!springcard_card)
	return FALSE;

  SCardLastError = SCardS10CardTransmit(SCardHandle, send_len, send_data, recv_len, recv_data, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return FALSE; 

  return TRUE; 
}

/*																STATUS : VALID
 *	Name : S10ReadDataArea
 *	----------------------
 *	S10 Read Function
 *---------------------------------------------------------------------------*/
BOOL S10ReadDataArea(BYTE *dest ,BYTE addr, BYTE len)
{ 
  BYTE status[2];

  SCardLastError = SCardS10ReadDataArea(SCardHandle, dest, addr, len, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return FALSE; 

  return TRUE; 
}

/*																STATUS : VALID
 *	Name : S10ReadProtectionArea
 *	----------------------------
 *	S10 Read Function
 *---------------------------------------------------------------------------*/
BOOL S10ReadProtectionArea(BYTE *dest)
{ 
  BYTE buffer[4];
  BYTE status[2];

  SCardLastError = SCardS10ReadProtectionArea(SCardHandle, buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return FALSE; 

  memcpy(dest, buffer, 4);

  return TRUE;
}

/*																STATUS : VALID
 *	Name : S10ReadSecurityArea
 *	--------------------------
 *	S10 Read Function
 *---------------------------------------------------------------------------*/
BOOL S10ReadSecurityArea(BYTE *dest)
{ 
  BYTE buffer[4];
  BYTE status[2];

  SCardLastError = SCardS10ReadSecurityArea(SCardHandle, buffer, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return FALSE; 

  memcpy(dest, buffer, 4);

  return TRUE;
}

/*																STATUS : VALID
 *	Name : S10WriteDataArea
 *	-------------
 *	S10 Write Function
 *---------------------------------------------------------------------------*/
BOOL S10WriteDataArea(BYTE *source ,BYTE addr, BYTE len)
{ 
  BYTE status[2];

  SCardLastError = SCardS10WriteDataArea(SCardHandle, source, addr, len, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return FALSE; 

  return TRUE;
}

/*																STATUS : VALID
 *	Name : S10WriteProtectionArea
 *	-------------
 *	S10 Write Function
 *---------------------------------------------------------------------------*/
BOOL S10WriteProtectionArea(BYTE *source ,BYTE addr, BYTE len)
{ 
  BYTE status[2];

  SCardLastError = SCardS10WriteProtectionArea(SCardHandle, source, addr, len, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return FALSE; 

  return TRUE;
}

/*																STATUS : VALID 
 *	Name : S10UpdateSecurityArea
 *	-------------
 *	S10 Write Function
 *---------------------------------------------------------------------------*/
BOOL S10UpdateSecurityArea(BYTE *source, BYTE addr, BYTE len)
{ 
  BYTE status[2];

  SCardLastError = SCardS10UpdateSecurityArea(SCardHandle, source, addr, len, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return FALSE; 

  return TRUE;
}

/*																STATUS : VALID
 *	Name : S10PresentCardSecretCode
 *	-------------
 *	S10PresentCardSecretCode
 *---------------------------------------------------------------------------*/
BOOL S10PresentCardSecretCode(BYTE code3, BYTE code2, BYTE code1)
{
  BYTE status[2];

  SCardLastError = SCardS10PSCVerification(SCardHandle, code3, code2, code1, status);
  if (SCardLastError != SCARD_S_SUCCESS)
    return FALSE; 

  return TRUE; 
}
