/*
  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.

  Copyright (c) 2000-2008 SpringCard - www.springcard.com
  See COPYING for copyright and legal info
  
  serial_palmos.h
  ---------------
  High level wrappers for RS232 communication under Palm OS
  
*/
#ifndef   __SERIAL_PALMOS_H__
#define   __SERIAL_PALMOS_H__

#include "sctypes.h"
#include "SerialMgrOld.h"

#define SER_SettingsType SerSettingsType
#define SER_SettingsPtr  SerSettingsPtr

/* Constants */
/* --------- */

#define SER_MaxBaud              115200L
#define	SER_DefaultBaud          9600L
#define SER_HandshakeThreshold   19200L
#define	SER_DefaultRecvQSize     0x200
#define SER_DefaultSendQSize     0x100
#define SER_DefaultSettings      0x00
#define SER_DefaultCTSTimeout    0x00

/* Structures */
/* ---------- */
typedef struct
{
  /* Receive/Transmit Queue. When the queue is empty, qStart == qEnd. */
  /* When the queue is full, end = start-1 (taking wrap-around        */
  /* into account if necessary).                                      */

  /* (start and end are declared as volatile because                  */
  /*  their values change at interrupt time)                          */
volatile WORD start;            // start offset of queue
volatile WORD end;              // end offset of queue
WORD    size;                   // size of queue
BYTE   *dataP;                  // pointer to queue data
BYTE   *defDataP;               // pointer to default queue data
}
SER_QueueType,
       *SER_QueuePtr;

typedef struct SERIAL_GlobalsType
{
SER_SettingsType settings;      // current settings
WORD    lineErrors;             // last errors bitfield

  /* Send and receive queues */
SER_QueueType sendQ;
SER_QueueType recvQ;

  /* Signalling information for the interrupt routine */
//  ProcPtr       wakeupHandler;    // Wakeup handler, if installed
//  WORD          wakeupRefcon;     // refcon for wakeup handler
WORD    sigSendFreeBytes;       // Desired # of free bytes in send buffer
WORD    sigRecvUsedBytes;       // How much the app is waiting for
DWORD   taskID;                 // which taskID is waiting on serial

  //DWord       lastRcvTicks;   // tick count of last received char
  //DWord       lastSendTicks;    // tick count of last received char


  // This flag gets set by our interrupt handler when we detect a <CR>
  // character come in while we're asleep. It tells us to wake up the
  // LCD the next time we get an interrupt and the system is finally awake. 
  //Boolean       wakeLCD;

  /* Hardware info */
struct
{
BOOL    isrInstalled;           // true if we successfully installed
                                 //  our ISR - used by SerHwFree()
WORD    cardNo;                 // card number
volatile BYTE * uartP;          // base address of UART
volatile BYTE * powerP;         // power-control toggle
volatile BYTE * resetP;         // reset-control toggle
BOOL    baudInited;             // false until baud rate setup
BOOL    modeInited;             // false until flags setup
BOOL    initFinished;           // false until int routine enabled

    /* Set by interrupt routine after it fills receive queue. Tells */
    /*  high level code to re-enable receive interrupts after       */
    /*  emptying receive buffer                                     */
BOOL    needRecvIntEnable;

BOOL    breakOn;                // set when break signal is asserted,
                                 //  cleared when break signal is
                                 //  deasserted; the default state is
                                 //  cleared -- necessary because the
                                 //  16650 LinceControl register is
                                 //  write-only

    // Copy of registers on chip. Necessary because these registers are
    //  write only
struct
{
BYTE    fifoControl;            // saved FIFO control
}
reg;

}
hw;

BOOL    errDisplayPatched;      // set by PrvApplySystemPatches
                                  //  in SerLib.c if ErrDisplayFileLineMsg
                                  //  was patched

DWORD   irqCount;

}
SER_GlobalsType,
       *SER_GlobalsPtr;

/* Hardware specific functions */
/* --------------------------- */

// Init hardware for first time, called by SerOpen
WORD    SER_HwInit(SER_GlobalsPtr gP,
                   WORD cardNo);

// Undo hardware initialization, called by SerClose
WORD    SER_HwFree(SER_GlobalsPtr gP);

// Put hardware into sleep mode
WORD    SER_HwSleep(SER_GlobalsPtr gP);

// Take hardware out of sleep mode
WORD    SER_HwWake(SER_GlobalsPtr gP);

// Setup hardware for new configuration
WORD    SER_HwControl(SER_GlobalsPtr gP,
                      WORD op,
                      void *valueP,
                      void *valueLenP);

// Wait for transmit FIFO to empty
WORD    SER_HwSendFIFOWait(SER_GlobalsPtr gP,
                           BOOL checkTimeout,
                           SDWORD ctsTimeout);

// Test
void    SER_HwLedOn(SER_GlobalsPtr gP);
void    SER_HwLedOff(SER_GlobalsPtr gP);

/* SerLib like functions */
/* --------------------- */
#include <HostControl.h>
// SER_ForHostID(void)
HostID  SER_ForHostID(void);

// SerOpen
WORD    SER_Open(void);

// SerClose
WORD    SER_Close(void);

// SerSleep
WORD    SER_Sleep(void);

// SerWake
WORD    SER_Wake(void);

// SerGetSettings
WORD    SER_GetSettings(SER_SettingsPtr settingsP);

// SerSetSettings
WORD    SER_SetSettings(SER_SettingsPtr settingsP);

// SerGetStatusRI
WORD    SER_GetStatusRI(BOOL * riOnP);

// SerGetStatus
WORD    SER_GetStatus(BOOL * ctsOnP,
                      BOOL * dsrOnP);

// SerClearErr
WORD    SER_ClearErr(void);

// SerSend
DWORD   SER_Send(void *bufP,
                 DWORD count,
                 WORD * errP);

// SerSendWait
WORD    SER_SendWait(SDWORD timeout);

// SerSendCheck
WORD    SER_SendCheck(DWORD * numBytesP);

// SerSendFlush
WORD    SER_SendFlush(void);

// SerReveiveChar, SPECIFIC
WORD    SER_ReceiveChar(BYTE * result,
                        SDWORD timeout);

// SerReceive
DWORD   SER_Receive(void *rcvBufP,
                    DWORD count,
                    SDWORD timeout,
                    WORD * errP);

// SerReceiveWait
WORD    SER_ReceiveWait(DWORD bytes,
                        SDWORD timeout);

// SerReceiveCheck
WORD    SER_ReceiveCheck(DWORD * numBytesP);

// SerReceiveFlush
void    SER_ReceiveFlush(SDWORD timeout);

// SerSetReceiveBuffer
WORD    SER_SetReceiveBuffer(void *bufP,
                             WORD bufSize);

// SerControl
WORD    SER_Control(WORD op,
                    void *valueP,
                    WORD * valueLenP);

// LedOn, LedOff
void    SER_LedOn(void);
void    SER_LedOff(void);

void    SER_ShowIrqCount(void);


/* A faire fournir par les ressources de l'application */
/* --------------------------------------------------- */

#define MSG_INFO     2000
#define MSG_WARNING  2010
#define MSG_ERROR    2020
#define MSG_CONFIRM  2030

#define MSG_OK       1
#define MSG_CANCEL   2
#define MSG_OKCANCEL 3
#define MSG_YESNO    4

#define AlertInfoOKAlert                          2001
#define AlertWarningOKAlert                       2011
#define AlertErrorOKAlert                         2021

#define SerialNoCardErrorString                   1050  // "Avant d'utiliser l'application, vous devez insrer le module "SpringCard SmardCard reader for MobiPDA"."
#define SerialCommErrorString                     1051  // "Le port srie n'est pas accessible."

/* ATTENTION, verifier les ressources ! */

int     SER_Write(BYTE * buffer,
                  int buff_size);
int     SER_Read(BYTE * buffer,
                 int max_buff_size);
void    SER_EasyClose(void);
int     SER_EasyOpen(void);

#endif                                 // __SERIAL_H__
