/*
  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.

  Copyright (c) 2000-2008 SpringCard - www.springcard.com
  See COPYING for copyright and legal info

  serial_pose.c
  -------------
  Low level wrappers for RS232 communication on the Palm OS emulator

*/

#include "sctypes.h"

#ifdef __palmos__
#ifdef POSE

#include <PalmOS.h>
#include <SystemPublic.h>
#include <SerialMgrOld.h>
#include "serial_palmos.h"

static UInt16 refNum = 0;

/*
 *  SER_ForHostID(void)
 *----------------------
 * Envoid le Host Id Compatibility
 */
HostID SER_ForHostID(void)
{
  return hostIDPalmOSEmulator;
}

WORD SER_Open(void)
{
  WORD    rc;
  rc = SysLibFind("Serial Library", &refNum);
  if (!rc)
    return SerOpen(refNum, 0, 9600);
  else
    return rc;
}

WORD SER_Close(void)
{
  return SerClose(refNum);
}

WORD SER_Sleep(void)
{
  return 0;
}

WORD SER_Wake(void)
{
  return 0;
}

WORD SER_GetSettings(SER_SettingsPtr settingsP)
{
  return SerGetSettings(refNum, settingsP);
}

WORD SER_SetSettings(SER_SettingsPtr settingsP)
{
  return SerSetSettings(refNum, settingsP);
}

WORD SER_GetStatusRI(BOOL * riOnP)
{
  /* Nothing to do */
  return 0;
}

WORD SER_GetStatus(BOOL * ctsOnP, BOOL * dsrOnP)
{
  return SerGetStatus(refNum, ctsOnP, dsrOnP);
}

WORD SER_ClearErr(void)
{
  return SerClearErr(refNum);
}

DWORD SER_Send(void *bufP, DWORD count, WORD * errP)
{
  return SerSend(refNum, bufP, count, errP);
}

WORD SER_SendWait(SDWORD timeout)
{
  return SerSendWait(refNum, timeout);
}

WORD SER_SendFlush(void)
{
  return SerSendFlush(refNum);
}

DWORD SER_Receive(void *rcvBufP, DWORD count, SDWORD timeout, WORD * errP)
{
  return SerReceive(refNum, rcvBufP, count, timeout, errP);
}

WORD SER_ReceiveWait(DWORD bytes, SDWORD timeout)
{
  return SerReceiveWait(refNum, bytes, timeout);
}

WORD SER_ReceiveCheck(DWORD * numBytesP)
{
  return SerReceiveCheck(refNum, numBytesP);
}

void SER_ReceiveFlush(SDWORD timeout)
{
  SerReceiveFlush(refNum, timeout);
}

WORD SER_SetReceiveBuffer(void *bufP, WORD bufSize)
{
  return SerSetReceiveBuffer(refNum, bufP, bufSize);
}

WORD SER_Control(WORD op, void *valueP, WORD * valueLenP)
{
  return SerControl(refNum, op, valueP, valueLenP);
}

void SER_LedOn(void)
{
  // Nothing
}

void SER_LedOff(void)
{
  // Nothing
}

int SER_EasyOpen(void)
{
  WORD    rc;
  SerSettingsType com_settings;

  /* Ouvre l'acces au port */
  rc = SER_Open();
  if ((rc != 0) && (rc != serErrAlreadyOpen))
  {
/*    ShowMessage(MSG_ERROR + MSG_OK, SerialNoCardErrorString);*/
    return FALSE;
  }

  /* Rglages du port srie */
  com_settings.baudRate = 9600;
  com_settings.flags = (serSettingsFlagBitsPerChar8 | serSettingsFlagStopBits1 /*| serSettingsFlagRTSAutoM */ );
  com_settings.ctsTimeout = serDefaultCTSTimeout;

  if (SerSetSettings(refNum, &com_settings))
  {
    SER_EasyClose();
    return FALSE;
  }

  /* Vidage du buffer d'mission */
  if (SerSendFlush(refNum))
  {
    SER_EasyClose();
    return FALSE;
  }

  return TRUE;
}

void SER_EasyClose(void)
{
  SerClose(refNum);
}

int SER_Read(BYTE * buffer, int max_buff_size)
{
  /* Variables locales  */
  unsigned int i = 0;
  unsigned int tries_count = 1;
  WORD    com_last_error = 0;

  while (i < max_buff_size)
  {
    i += SerReceive(refNum, (BYTE *) & (buffer[i]), 1, 400, &com_last_error);
    if (com_last_error)
    {
      SerClearErr(refNum);
      return i;
    }
  }

  return i;
}

int SER_Write(BYTE * buffer, int buff_size)
{
  WORD    com_last_error;

  SerClearErr(refNum);
  SerSend(refNum, buffer, buff_size, &com_last_error);
  if (!com_last_error)
    com_last_error = SerSendWait(refNum, 200);

  if (com_last_error != 0)
    return -1;
  else
    return buff_size;
}

#endif
#endif
