/*
  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.

  Copyright (c) 2000-2008 SpringCard - www.springcard.com
  See COPYING for copyright and legal info
  
  serial_visor.h
  ---------------
  Low level wrappers for RS232 communication on the Visor
  
*/

/*
 * This file is based on Ser166550.h, the Serial 16650 Library for
 * HandSpring Visor featuring a compatible UART on its SpringBoard
 * slot.
 */

/*
 *
 * Project:
 *	  Serial 16650 Library
 *
 * Copyright info:
 *
 *	  This is free software; you can redistribute it and/or modify
 *	  it as you like.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 *
 * FileName:
 *	  Ser16650Prv.h
 *
 * Description:
 *	  Sample removable card library for the Handspring Developer
 *	  Reference Card.  This library provides an interface to the
 *	  ST16C650A UART on the card and is API conpatible with the 
 *	  PalmOS Serial Library.  
 *
 *	  This file contains the hardware specific equates for the 16650
 *	  UART on the card. 
 *
 * History:
 *	  1-Mar-1999		Created by Ron Marianetti  
 *
 *******************************************************************/

#ifndef   _SERIAL_VISOR_H_
#define   _SERIAL_VISOR_H_

#include "sctypes.h"
#include "serial_palmos.h"

#define OutOfMemoryAlert                          1000

//**********************************************************************
//  Constants
//**********************************************************************

// Offset from 2nd chip select to start of UART registers
// The UART is decoded at csSlot1 + 0x00200000 and it's registers
//  start at (0x3F8 * 2) and are every other byte
#define	SER_HwUartOffset  0x2007F0
#define	SER_HwPowerOffset 0x800000
#define	SER_HwResetOffset 0xC00000

// Name of card with our UART on it
#define	  SER_HwCardName		"HsSerRefModule"

// Version required
#define	  SER_HwCardVersion	0x0100

// Clock frequency

#define	  SER_HwClockFreq1	  7372800
#define   SER_HwClockFreq2   14745600

//**********************************************************************
//  Structures
//**********************************************************************

// For reading
typedef volatile struct
{
BYTE    recData;                // 000:   7F0: receive data register
BYTE    _filler1;               // 

WORD    _filler2;               // 001:   7F2:

BYTE    intStatus;              // 010:   7F4: interrupt status
BYTE    _filler3;

WORD    _filler4;               // 011:   7F6:

WORD    _filler5;               // 100:   7F8:

BYTE    lineStatus;             // 101:   7FA: line status
BYTE    _filler6;

BYTE    modemStatus;            // 110:   7FC: modem status
BYTE    _filler7;

BYTE    scratch;                // 111:   7FE: scratch pad
BYTE    _filler8;

}
SER_Hw16650ReadType;

// For writing
typedef volatile struct
{
BYTE    sendData;               // 000:   7F0: send data 
BYTE    _filler1;

BYTE    intEnable;              // 001:   7F2: interrupt enable 
BYTE    _filler2;

BYTE    fifoControl;            // 010:   7F4: FIFO control 
BYTE    _filler3;

BYTE    lineControl;            // 011:   7F6: Line Control
BYTE    _filler4;

BYTE    modemControl;           // 100:   7F8: Modem Control
BYTE    _filler5;

WORD    _filler6;               // 101:   7FA:

WORD    _filler7;               // 110:   7FC:

BYTE    scratch;                // 111:   7FE: scratch pad
BYTE    _filler8;

}
SER_Hw16650WriteType;


// Baud registers, accessed when 
//    (SER_Hw16650WriteType->lineControl & 0x80)
// The baud rate is: (SER_HwClockFreq / 16 / baudReg)
typedef volatile struct
{
BYTE    baudLSB;                // 000:   low byte
BYTE    _filler1;

BYTE    baudMSB;                // 001:   high byte
BYTE    _filler2;

}
SER_Hw16650BaudType;


// Enhanced fature set
typedef volatile struct
{
WORD    _filler1;               // 000:

WORD    _filler2;               // 001:

BYTE    enhReg;                 // 010: 7F4: enhanced registers
BYTE    _filler3;

WORD    _filler4;               // 011: 7F6:

BYTE    xon1;                   // 100  7F8: XON 1
BYTE    _filler5;

BYTE    xon2;                   // 101  7F8: XON 2
BYTE    _filler6;

BYTE    xoff1;                  // 110  7F8: XOFF 1
BYTE    _filler7;

BYTE    xoff2;                  // 111  7F8: XOFF 2
BYTE    _filler8;

}
SER_Hw16650EnhType;


// ===============================================================
// Register bits
// ===============================================================

// LineControl register
#define	SER_Hw16650LineCtlData0		0x01
#define	SER_Hw16650LineCtlData1		0x02
#define	SER_Hw16650LineCtlStop		0x04
#define	SER_Hw16650LineCtlParityEn	0x08
#define	SER_Hw16650LineCtlParityEven	0x10
#define	SER_Hw16650LineCtlParitySet	0x20
#define	SER_Hw16650LineCtlBreakSet	0x40
#define	SER_Hw16650LineCtlDivLatchEn	0x80

#define	SER_Hw16650LineCtlEnFeatures	0xBF


// ModemControl register
#define	SER_Hw16650ModemCtlDTR		0x01
#define	SER_Hw16650ModemCtlRTS		0x02
#define	SER_Hw16650ModemCtlLED		0x04
#define	SER_Hw16650ModemCtlIRQx		0x08
#define	SER_Hw16650ModemCtlLoopback	0x10
#define	SER_Hw16650ModemCtlIREnable	0x40
#define	SER_Hw16650ModemCtlClockSel	0x80


// Enhanced Features Register
#define	SER_Hw16650EnhEnableAdv		0x10
#define	SER_Hw16650EnhSpecialChar	0x20
#define	SER_Hw16650EnhAutoRTS		0x40
#define	SER_Hw16650EnhAutoCTS		0x80

// Modem Status register
#define	SER_Hw16650ModemStDeltaCTS	0x01
#define	SER_Hw16650ModemStDeltaRTS	0x02
#define	SER_Hw16650ModemStDeltaRI	0x04
#define	SER_Hw16650ModemStDeltaCD	0x08
#define	SER_Hw16650ModemStCTS		0x10
#define	SER_Hw16650ModemStDSR		0x20
#define	SER_Hw16650ModemStRI			0x40
#define	SER_Hw16650ModemStCD			0x80

// Interrupt Enable register
#define	SER_Hw16650IntEnReceive		0x01
#define	SER_Hw16650IntEnTransmit		0x02
#define	SER_Hw16650IntEnRcvStatus	0x04
#define	SER_Hw16650IntEnModemStatus	0x08
#define	SER_Hw16650IntEnSleep	  	0x10
#define	SER_Hw16650IntEnXOff			0x20
#define	SER_Hw16650IntEnRTS			0x40
#define	SER_Hw16650IntEnCTS			0x80

// Line Status Register
#define	SER_Hw16650LineStReceive		0x01
#define	SER_Hw16650LineStOverrun		0x02
#define	SER_Hw16650LineStParity		0x04
#define	SER_Hw16650LineStFraming		0x08
#define	SER_Hw16650LineStBreak		0x10
#define	SER_Hw16650LineStTransHold	0x20
#define	SER_Hw16650LineStTransmit	0x40
#define	SER_Hw16650LineStFIFOErr		0x80


// Fifo Control Register (FCR)
#define SER_Hw16650FifoCtlRxTrigMask	0xC0  // Receive data ready interrupt
#define SER_Hw16650FifoCtlRxTrig8	0x00  //  is generated when the # of
#define SER_Hw16650FifoCtlRxTrig16	0x40  //  chars in the RX FIFO reaches
#define SER_Hw16650FifoCtlRxTrig24	0x80  //  the programmed trigger level
#define SER_Hw16650FifoCtlRxTrig28	0xC0

#define SER_Hw16650FifoCtlTxTrigMask	0x30  // Transmit Empty interrupt is
#define SER_Hw16650FifoCtlTxTrig16	0x00  //  generated when the # of chars
#define SER_Hw16650FifoCtlTxTrig8	0x10  //  in the TX FIFO drops below the
#define SER_Hw16650FifoCtlTxTrig24	0x20  //  selected trigger level
#define SER_Hw16650FifoCtlTxTrig30	0x30

#define SER_Hw16650FifoCtlDMA		0x08  // set = DMA ON; clear = OFF
#define SER_Hw16650FifoCtlTxReset	0x04  // setting resets transmit FIFO
#define SER_Hw16650FifoCtlRxReset	0x02  // setting resets receive FIFO
#define SER_Hw16650FifoCtlFIFOEnable	0x01  // setting enables FIFO mode


// FIFO sizes
#define SER_Hw16650RxFIFOSize		32  // size of receive FIFO
#define SER_Hw16650TxFIFOSize		32  // size of transmit FIFO

/* Internal control codes that we define as an interface to */
/*  our SER_HwSetSetting call                                */
#define SER_CtlFirstCustom	0xA800  // This should have been defined by Palm
                                  //  in SerMgr.h, but wasn't at the time
                                  //  of this writing
typedef enum
{
SER_HwCtlBaudRateSet = SER_CtlFirstCustom,  // set baud rate, valueP = &DWord, *valueLenP = sizeof (DWord);
SER_HwCtlFlagsSet,              // SerSettingsType.flags, valueP = &DWord, *valueLenP = sizeof (DWord);
SER_HwCtlCTS,                   // State of CTS input, valueP = &Byte, *valueLenP = sizeof (Byte);
SER_HwCtlDSR,                   // State of DSR input, valueP = &Byte, *valueLenP = sizeof (Byte);
SER_HwCtlRecvIntEnable,
                                             // Re-enable receive interrupts, valueP  unused, valueLenP  unused
SER_HwCtlSendIntEnable,
                                             // Re-enable send interrupts, valueP  unused, valueLenP  unused
SER_HwCtlLast,                  // Leave at end
SER_HwCtlRI
}
SER_HwCtlEnum;

// void SER_HwIntHandler(DWORD cardParam, BOOL *sysAwakeP);

void    SER_Hw5VOn(SER_GlobalsPtr gP);

void    SER_Hw5VOff(SER_GlobalsPtr gP);

#endif
