/*
  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.

  Copyright (c) 2000-2008 SpringCard - www.springcard.com
  See COPYING for copyright and legal info
  
  serial_win32.c
  --------------
  Low level wrappers for RS232 communication on the Win32 platform
  (i.e. desktop Windows and Windows CE)
  
*/

/*
 * This file is based on serial.c provided by GemPlus as part of the TLP
 * library
 */

/*****************************************************************************
*               Copyright (c) Gemplus Card International 1997
*
* Name        : serial.c
*
* Description : Homogenization of read-write function parameters

  Author      : L.GAUTERON
  Compiler    : Visual C++
  Host        : Windows 9x,NT
*
* $Id: serial.c,v 1.3 2000/12/11 17:53:30 rousseau Exp $
* $Log: serial.c,v $
* Revision 1.3  2000/12/11 17:53:30  rousseau
* Added a 5 seconds timeout if no reader present
*
* Revision 1.2  2000/09/04 08:19:06  rousseau
* code cleanup, changed the API and name for [Init|Close]Serial
*
*
******************************************************************************
*
* Warning     : 
*
* Remarks     : 
*
*****************************************************************************/

#ifdef WIN32

#include "springcard_i.h"

/* To remember which port to use */
TCHAR   sc_com_name[16] = _T("");
#ifndef UNDER_CE
  /* "Regular" Windows, be careful it is "COMx" without the ":" */
#define COM_NAME _T("COM%d")
#else
  /* Windows CE, be careful it is "COMx:" with a ":" */
#define COM_NAME _T("COM%d:")
#endif

#define TMO_INTER        100
#define TMO_LONG_TRANS   1000
#define TMO_SHORT_TRANS  100

#define MAX_FRAME_LEN    32

/*****************************************************************************
 *
 * Serial
 *
 *****************************************************************************/

/********************************************/
int WriteCom(HANDLE fd, LPCVOID lpBuffer, DWORD dwNbByte)
{
  DWORD   dwWritten;
  DWORD   dwNbToWrite;
  BYTE   *p;

  SCARD_Trace(TR_COMM_LOW, "WriteFile NbByte=%d (Handle=%08lX)", dwNbByte, fd);

  if (fd == INVALID_HANDLE_VALUE)
    return -1;

  p = (BYTE *) lpBuffer;

  while (dwNbByte)
  {
    if (dwNbByte < MAX_FRAME_LEN)
      dwNbToWrite = dwNbByte;
    else
      dwNbToWrite = MAX_FRAME_LEN;

    if (!WriteFile(fd, p, dwNbToWrite, &dwWritten, 0))
    {
      SCARD_Trace(TR_COMM_LOW, "WriteFile(%d->%d) error (%d)", dwNbToWrite, dwWritten, GetLastError());
      switch (GetLastError())
      {
        case ERROR_INVALID_HANDLE:
        case ERROR_INVALID_HANDLE_STATE:
          return -2;
      }
      return -1;
    }

    if (dwWritten < dwNbToWrite)
    {
      SCARD_Trace(TR_COMM_LOW, "WriteFile(%d/%d) failed (%d)", dwWritten, dwNbToWrite, GetLastError());
      return -1;
    }

    p += dwWritten;
    dwNbByte -= dwWritten;
  }

  return (int) dwWritten;
}

/********************************************/

/********************************************/
int ReadCom(HANDLE fd, LPVOID lpBuffer, DWORD dwNbByte)
{
  DWORD   dwRead;
  DWORD   dwWantRead;
  DWORD   dwNbToRead;
  DWORD   dwTotalRead = 0;
  BYTE   *p;

 SCARD_Trace(TR_COMM_LOW, "ReadFile NbByte=%d (Handle=%08lX)", dwNbByte, fd);

  if (fd == INVALID_HANDLE_VALUE)
    return -1;

  p = (BYTE *) lpBuffer;
  dwWantRead = dwNbByte;
  
  while (dwWantRead)
  {
    if (dwWantRead < MAX_FRAME_LEN)
      dwNbToRead = dwWantRead;
    else
      dwNbToRead = MAX_FRAME_LEN;
    
    if (!ReadFile(fd, p, dwNbToRead, &dwRead, 0))
    {
      SCARD_Trace(TR_COMM_LOW, "ReadFile(%d->%d) failed (%d)", dwNbToRead, dwRead, GetLastError());
      switch (GetLastError())
      {
        case ERROR_INVALID_HANDLE:
        case ERROR_INVALID_HANDLE_STATE:
          return -2;
      }
      return -1;
    }

    if (dwRead == 0)
      break;

    p += dwRead;
    dwWantRead -= dwRead;
    dwTotalRead += dwRead;
  }

  if (dwTotalRead < dwNbByte)
  {
    SCARD_Trace(TR_COMM_LOW, "ReadFile : read %d, %d expected", dwTotalRead, dwNbByte);
  }

  return dwTotalRead;
}

/********************************************/

/********************************************
 * SerialFlush
********************************************/
int SerialFlush(HANDLE fd)
{
  DWORD   dw_dummy;
  COMSTAT cs_dummy;

  if (fd == INVALID_HANDLE_VALUE)
    return -1;

  SCARD_Trace(TR_COMM_LOW, "Flush...");

  FlushFileBuffers(fd);
  ClearCommError(fd, &dw_dummy, &cs_dummy);
  PurgeComm(fd, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);

  return 0;
}

/********************************************/

/********************************************
 * SerialConfig
********************************************/
int SerialConfig(HANDLE fd, DWORD baudrate)
{
  DCB     dcb;

  if (fd == INVALID_HANDLE_VALUE)
    return -1;

  /* Retrieve serial port data */
  if (!GetCommState(fd, &dcb))
  {
    SCARD_Trace(TR_COMM_LOW, "SerialConfig:GetCommState failed (%d)", GetLastError());
    return -1;
  }
  
  SCARD_Trace(TR_COMM_LOW, "SerialConfig: selecting %ldbps", baudrate);

  /* Configure serial port hardware */
  dcb.BaudRate = baudrate;
  dcb.fBinary = TRUE;
  dcb.fParity = FALSE;
  dcb.fOutxCtsFlow = FALSE;
  dcb.fOutxDsrFlow = FALSE;
  dcb.fDsrSensitivity = FALSE;
  dcb.fOutX = FALSE;
  dcb.fInX = FALSE;
  dcb.fNull = FALSE;
  dcb.fAbortOnError = FALSE;
  dcb.ByteSize = 8;
  dcb.Parity = NOPARITY;
  dcb.StopBits = ONESTOPBIT;
  dcb.fRtsControl = RTS_CONTROL_DISABLE;
  dcb.fDtrControl = DTR_CONTROL_DISABLE;

  if (!SetCommState(fd, &dcb))
  {
    SCARD_Trace(TR_COMM_LOW, "SerialConfig:SetCommState failed (%d)", GetLastError());
    return -1;
  }

  return 0;
}

/********************************************/

/********************************************
 * SerialSetTimeouts
********************************************/
static int SerialTimeouts(HANDLE fd, DWORD interval, DWORD total)
{
  COMMTIMEOUTS stTime;

  if (fd == INVALID_HANDLE_VALUE)
    return -1;

  stTime.ReadIntervalTimeout = interval;
  stTime.ReadTotalTimeoutMultiplier = interval;
  stTime.ReadTotalTimeoutConstant = total;

  stTime.WriteTotalTimeoutMultiplier = interval;
  stTime.WriteTotalTimeoutConstant = total;
  
  SCARD_Trace(TR_COMM_HIGH, "SerialTimeouts:ReadIntervalTimeout=ReadTotalTimeoutMultiplier=WriteTotalTimeoutMultiplier=%ld", interval);
  SCARD_Trace(TR_COMM_HIGH, "SerialTimeouts:ReadTotalTimeoutConstant=WriteTotalTimeoutConstant=%ld", total);
  
  if (!SetCommTimeouts(fd, &stTime))
  {
    SCARD_Trace(TR_COMM_LOW, "SerialTimeouts:SetCommTimeouts failed (%d)", GetLastError());
    return -1;
  }

  SCARD_Trace(TR_COMM_LOW, "Timeout set to interval=%ld, total=%ld", interval, total);

  return 0;
}

/********************************************/

/********************************************
 * SerialClose
********************************************/
void SerialClose(ReaderDesc * reader)
{
  SCARD_Trace(TR_COMM_LOW, "SerialClose:Handle=%08lX", reader->fd);
  
  if (reader->fd == INVALID_HANDLE_VALUE)
    return;

  SerialFlush(reader->fd);

  SCARD_Trace(TR_COMM_LOW, "Closing reader...");
  CloseHandle(reader->fd);

  reader->fd = INVALID_HANDLE_VALUE;
}

/********************************************/

/********************************************
 * SerialOpen
********************************************/
int SerialOpen(ReaderDesc * reader, TCHAR * device)
{
  HANDLE  fd;

  SCARD_Trace(TR_COMM_LOW, "SerialOpen:%s", device);

  fd = CreateFile((LPCTSTR) device, GENERIC_READ | GENERIC_WRITE, 0,  // comm devices must be opened w/exclusive-access
                  NULL,         // no security attributes
                  OPEN_EXISTING,  // comm devices must use OPEN_EXISTING
                  0,            // not overlapped I/O
                  NULL          // hTemplate must be NULL for comm devices
    );
    
  SCARD_Trace(TR_COMM_LOW, "SerialOpen:Handle of CreateFile=%08lX", fd);

  if (fd == INVALID_HANDLE_VALUE)
  {
    SCARD_Trace(TR_COMM_LOW, "SerialOpen:CreateFile failed (%d)", GetLastError());
    return -1;
  }

  /* Configure the device (9600bps is default speed) */
  if (SerialConfig(fd, NORM_BAUDRATE))
  {
    SCARD_Trace(TR_COMM_LOW, "SerialOpen:SerialConfig failed");
    SCARD_Trace(TR_COMM_LOW, "SerialOpen:Handle of CloseHandle=%08lX", fd);
    CloseHandle(fd);
    return -1;
  }

  reader->fd = fd;

  /* Give some time for the GemCore to be ready after power-up */
  Sleep(50);

  return 0;
}

/********************************************/

/*****************************************************************************
 *
 * Reader
 *
 *****************************************************************************/

/********************************************
 * ResetReader
 *******************************************/
static int ResetReader(ReaderDesc * reader)
{
  BYTE    cmd[4] = { 0x0C, 0x00, 0x00, 0x00 };

  if (reader->fd == INVALID_HANDLE_VALUE)
    return -1;

  if (WriteGBP(reader, sizeof(cmd), cmd) != GC_STATUS_SUCCESS)
  {
    SCARD_Trace(TR_COMM_LOW, "ResetReader:WriteGBP failed");
    return -1;
  }

  SerialFlush(reader);

  return 0;
}

/********************************************/

/********************************************
 * SetReaderSpeed
 *******************************************/
static int SetReaderSpeed(ReaderDesc * reader, DWORD baudrate)
{
  if (reader->fd == INVALID_HANDLE_VALUE)
    return -1;

  // Don't care for the return code
  GBPSetSIOSpeed(reader, baudrate);

  return 0;
}

/********************************************/

/********************************************
 * InitReader
 *******************************************/
int InitReader(ReaderDesc * reader)
{
  if (reader->fd == INVALID_HANDLE_VALUE)
    return -1;

#ifdef GCR_HIGH_BAUD_RATE
  /* Enter high speed mode */
  SCARD_Trace(TR_COMM_LOW, "HighBaudrate !");
  if (SetReaderSpeed(reader, HIGH_BAUDRATE))
    return -1;

//    SerialFlush(reader);

  if (SerialConfig(reader, HIGH_BAUDRATE))
    return -1;
#else
  /* Confirm default speed */
//  SCARD_Trace(TR_COMM_LOW, "SlowBaudrate !");
//  if (SetReaderSpeed(reader, NORM_BAUDRATE))
//    return -1;
//  SerialFlush(reader);
#endif

  return 0;
}

/********************************************/

LONG RDR_OpenEx(SCARD_CONTEXT_ST *context, TCHAR * port, DWORD reader_specific_info)
{
  LONG    rc;

  memset(&context->desc, 0, sizeof(context->desc));

  if (SerialOpen(&context->desc, port) != 0)
  {
    SCARD_Trace(TR_COMM_LOW, "SerialOpen failed");
    return SCARD_E_UNKNOWN_READER;
  }

  /* Assume device has 2 slots */
  context->slot_count = SLOT_COUNT;

  /* Check port name to determine slot count */
  if ((reader_specific_info & DEVICE_INFO_TYPE) == DEVICE_INFO_TYPE_SD)
  {
    /* Reader opened on SDIO port : this must be the SpringSim-SD (1 slot only) */
    context->slot_count = 1;
  }

  /* Init reader */
  if (InitReader(&context->desc) != 0)
  {
    SCARD_Trace(TR_COMM_LOW, "InitReader failed");
    SerialClose(&context->desc);
    return SCARD_E_READER_UNAVAILABLE;
  }

  /* Read firmware */
  SerialTimeouts(context->desc.fd, TMO_INTER, TMO_SHORT_TRANS);
  rc = RDR_GetFirmware(context);
  if (rc != SCARD_S_SUCCESS)
  {
    SCARD_Trace(TR_COMM_LOW, "GetFirmware failed at 9600bps - Trying again at 38400bps");

    SerialFlush(context->desc.fd);
    SerialConfig(context->desc.fd, 38400); 
    SerialFlush(context->desc.fd);  

    if (RDR_GetFirmware(context) != SCARD_S_SUCCESS)    
    {
      SCARD_Trace(TR_COMM_LOW, "GetFirmware failed again at 38400bps - Aborting");
      SerialClose(&context->desc);    
      return rc;
    }
    
    rc = SCARD_S_SUCCESS;
  } else
  {
    /* Go to 38400bps */
    rc = RDR_HighSpeed(context);
    if (rc != SCARD_S_SUCCESS)
    {
      SCARD_Trace(TR_COMM_LOW, "Going to 38400bps failed - Aborting");
      SerialClose(&context->desc);
      return rc;
    }
  }

  /* OK reader succesfully opened */
  SerialTimeouts(context->desc.fd, TMO_INTER, TMO_LONG_TRANS);
  SCARD_Trace(TR_COMM_LOW, "Reader active OK !");
  return rc;
}

/********************************************/

/********************************************
 * reader_on
********************************************/
LONG RDR_Open(SCARD_CONTEXT_ST *context)
{
#ifndef UNDER_CE
  DWORD   i;
#endif
  DWORD   reader_specific_info = 0;
  LONG    rc;

  if (context->enabled)
    return SCARD_S_SUCCESS;

#ifndef UNDER_CE

  /* Try to reopen previously open port */
  /* ---------------------------------- */
  if (_tcslen(sc_com_name))
  {
    rc = RDR_OpenEx(context, sc_com_name, reader_specific_info);
    if (rc == SCARD_S_SUCCESS)
      goto ok;
  }

  /* Regular PC : browse up to 9 serial ports */
  /* ---------------------------------------- */
  for (i = 1; i <= 9; i++)
  {
    _stprintf(sc_com_name, COM_NAME, i);
    rc = RDR_OpenEx(context, sc_com_name, reader_specific_info);
    if (rc == SCARD_S_SUCCESS)
      goto ok;
  }

#else

  /* Pocket PC : lookup for the SpringCard key in the registry */
  /* --------------------------------------------------------- */

  if (GetDefaultDevice(sc_com_name, sizeof(sc_com_name) / sizeof(TCHAR), &reader_specific_info))
  {
    /* Trying to open */
    rc = RDR_OpenEx(context, sc_com_name, reader_specific_info);
    /* OK !!! */
    if (rc == SCARD_S_SUCCESS)
      goto ok;
    return SCARD_E_READER_UNAVAILABLE;
  }
#if 0
  /* Not found... Maybe we can try to browse the 9 serial ports */
  for (i = 1; i <= 9; i++)
  {
    _stprintf(sc_com_name, COM_NAME, i);
    rc = RDR_OpenEx(context, sc_com_name, reader_specific_info);
    if (rc == SCARD_S_SUCCESS)
      goto ok;
  }
#endif

#endif

  return SCARD_E_READER_UNAVAILABLE;  /* Reader not found, simply... */

ok:
  context->enabled = TRUE;
  context->start_count = 1;
  return SCARD_S_SUCCESS;
}

/********************************************/

#endif
