#ifndef _SPRINGCARD_I_H_
#define _SPRINGCARD_I_H_

#ifdef GCR_IS_LITE
	#error Deprecated define
#endif

/* SpringCard library release */
/* -------------------------- */
#include "version.h"

#ifdef WIN32
  /* Prevent future inclusion of winscard.h */
  #ifdef _WINSCARD_H_
    #error winscard.h included !
  #endif
  #define _WINSCARD_H_
  #include <windows.h>
  #include <tchar.h>
  /* We are building the library */
  #define SPRINGCARD_LIB __declspec( dllexport )
#endif

#ifndef __palmos__
  extern TCHAR sc_com_name[16];
#endif

#include "reader.h"

#define SLOT_COUNT 2

#define GCR_SLOT_0 0
#define GCR_SLOT_1 1

#define INVALID_CONTEXT (SCARDCONTEXT) INVALID_HANDLE_VALUE
#define INVALID_CARD    (SCARDHANDLE)  INVALID_HANDLE_VALUE

typedef struct
{
  void   *context;
  DWORD   status;
  DWORD   ask_protocol;
  DWORD   run_protocol;
  BYTE    atr[32];
  BYTE    atr_len;
  BYTE    slot;
  DWORD   timeout;
} SCARD_HANDLE_ST;
#define SCARD_HANDLE_T SCARD_HANDLE_ST *

typedef struct
{
  BOOL    enabled;
  BYTE    start_count;
  BYTE    slot_count;
  void   *cards[SLOT_COUNT];
  char    names[33 * SLOT_COUNT];
  ReaderDesc desc;
  LONG    last_error;
  #ifdef WIN32
  HANDLE  mutex;
  #else
  BOOL    mutex;
  #endif
} SCARD_CONTEXT_ST;
#define SCARD_CONTEXT_T SCARD_CONTEXT_ST *

#include "gbp_impl.h"
#include "pcsc_impl.h"

#undef _WINSCARD_H_
#include "springcard.h"

/*
 *             EMV    SYN
 * ORIGINAL            X
 * LITE         X
 * LITE PRO     X
 * POS
 */

extern BYTE gcr_family;
extern const TCHAR *gcr_names[3];
extern TCHAR sc_com_name[16];

BYTE SCARD_TraceGetLevel(void);
void SCARD_TraceSetLevel(BYTE level);
void SCARD_TraceSetFile(const TCHAR *filename);
void SCARD_Trace(BYTE level, const char *fmt, ...);

/*
 * From pcsc_low.c
 * ---------------
 */
LONG SCARD_PowerUp_Raw(SCARD_HANDLE_ST *card);
LONG SCARD_PowerUp(SCARD_HANDLE_ST *card);
LONG SCARD_PowerDown(SCARD_HANDLE_ST *card);
LONG SCARD_Transmit(SCARD_HANDLE_ST *card, BYTE *sendBuffer, DWORD sendLen, BYTE *recvBuffer, DWORD recvSize, DWORD *recvLen);
LONG SCARD_Status(SCARD_HANDLE_ST *card);
LONG SCARD_Control(SCARD_HANDLE_ST *card, BYTE *sendBuffer, DWORD sendLen, BYTE *recvBuffer, DWORD recvSize, DWORD *recvLen);

LONG RDR_GetFirmware(SCARD_CONTEXT_ST *context);
LONG RDR_HighSpeed(SCARD_CONTEXT_ST *context);
LONG RDR_Resume(SCARD_CONTEXT_ST *context);
LONG RDR_Suspend(SCARD_CONTEXT_ST *context);
LONG RDR_Open(SCARD_CONTEXT_ST *context);
void RDR_Close(SCARD_CONTEXT_ST *context);


/*
 * From gcr_high.c
 * ---------------
 */
LONG GCR_Command(SCARD_HANDLE_ST *card, BYTE *sendBuffer, DWORD sendLen, BYTE *recvBuffer, DWORD recvSize, DWORD *recvLen, DWORD timeout);
LONG GCR_PowerUp(SCARD_HANDLE_ST *card);
LONG GCR_PowerDown(SCARD_HANDLE_ST *card);
LONG GCR_PowerUpEx(SCARD_HANDLE_ST *card, BYTE cfg);
LONG GCR_SetMode(SCARD_HANDLE_ST *card, BYTE mode);
LONG GCR_SetType(SCARD_HANDLE_ST *card, BYTE type);
LONG GCR_Status(SCARD_HANDLE_ST *card);
LONG GCR_Transmit(SCARD_HANDLE_ST *card, BYTE *sendBuffer, DWORD sendLen, BYTE *recvBuffer, DWORD recvSize, DWORD *recvLen);
LONG GCR_GetFirmware(ReaderDesc * reader, BYTE buffer[], DWORD * length);
LONG GCR_SetBaudrate(ReaderDesc * reader, DWORD baudrate);

/*
 * From gcr_low.c
 * --------------
 */
LONG GBP_Command(ReaderDesc * reader, DWORD nLengthIn, const BYTE pcBufferCmd[], DWORD * pnLengthOut, BYTE pcBufferOut[]);


#define TR_COMM_LOW  0x01
#define TR_COMM_HIGH 0x02
#define TR_GCR_LOW   0x04
#define TR_GCR_HIGH  0x08
#define TR_PCSC_LOW  0x10
#define TR_PCSC_HIGH 0x20
#define TR_CARD_LOW  0x40
#define TR_CARD_HIGH 0x80

#ifdef WIN32
  #define SCARD_INLINE __forceinline
#else
  #define SCARD_INLINE static
#endif

#include "c2bin.h"
#include "bin2c.h"

#include "config_win32.h"
#include "serial_win32.h"

#ifdef WIN32
  #ifndef UNDER_CE
    #ifndef UNICODE
      #define ADD_UNICODE_SUPPORT
    #endif
  #endif
#endif

#endif
