/*
  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.

  Copyright (c) 2000-2008 SpringCard - www.springcard.com
  See COPYING for copyright and legal info
  
  springcard.h
  ------------
  Defines, prototypes of exported functions

*/

#ifndef _SPRINGCARD_H_
#define _SPRINGCARD_H_

#ifdef WIN32
  /* Prevent future inclusion of winscard.h */
  #ifdef _WINSCARD_H_
    #error winscard.h included !
  #endif
  #define _WINSCARD_H_
  #include <windows.h>
#endif



#ifdef __MC68K__
  /* Hack for CodeWarrior PalmOS build */
  #define __palmos__
#endif


#ifndef WIN32
  /* Define basic types for non-Win platform */
  /* --------------------------------------- */
  typedef unsigned char BOOL;
  #define FALSE 0
  #define TRUE 1
  typedef unsigned char BYTE;
  typedef BYTE *LPBYTE;
  typedef unsigned short WORD;
  typedef WORD *LPWORD;
  typedef unsigned long DWORD;
  typedef DWORD *LPDWORD;
  typedef signed long LONG;
  typedef char TCHAR;
  typedef const void *LPCVOID;
  typedef void *LPVOID;
  typedef const TCHAR *LPCTSTR;
  typedef TCHAR *LPTSTR;
#endif

/* SCARD context & handles are abstract data types */
/* ----------------------------------------------- */
#ifndef SCARD_CONTEXT_T 
  #define SCARD_CONTEXT_T void *
#endif
#ifndef SCARD_HANDLE_T
  #define SCARD_HANDLE_T void *
#endif

/* Remain compliant with earlier release... */
typedef SCARD_CONTEXT_T SCARDCONTEXT;
typedef SCARD_HANDLE_T  SCARDHANDLE;

/* Intersting defines... */
#define MAX_ATR_SIZE                    33  /* Maximum ATR size */
#define MAX_READERNAME                  50  /* 50 char Reader Name */
#define MAX_LIBNAME                     100 /* 100 char Lib Name */
#define MAX_BUFFER_SIZE                 264 /* Maximum Tx/Rx Buffer */

/* GCR Family */
#define GCR_ORIGINAL 10
#define GCR_LITE     13
#define GCR_LITE_PRO 14
#define GCR_POS      15

typedef struct
{
  const char *szReader;
  void   *pvUserData;
  unsigned long dwCurrentState;
  unsigned long dwEventState;
  unsigned long cbAtr;
  unsigned char rgbAtr[MAX_ATR_SIZE];
} SCARD_READERSTATE_A;

typedef SCARD_READERSTATE_A SCARD_READERSTATE, *PSCARD_READERSTATE_A, *LPSCARD_READERSTATE_A;

typedef struct _SCARD_IO_REQUEST
{
  unsigned long dwProtocol;       /* Protocol identifier */
  unsigned long cbPciLength;      /* Protocol Control Inf Length */
} SCARD_IO_REQUEST, *PSCARD_IO_REQUEST, *LPSCARD_IO_REQUEST;

typedef const SCARD_IO_REQUEST *LPCSCARD_IO_REQUEST;

/*
 * List of error codes
 * -------------------
 */
#ifndef SCARD_S_SUCCESS
  /* On Windows use MS-defined error codes instead */

  #ifdef __palmos__
    /* 16-bit error codes */
    #define SCARD_E_PREFIX 0x0000
  #else
    /* 32-bit error codes */
    #define SCARD_E_PREFIX 0x80100000
  #endif

  #define SCARD_S_SUCCESS                               0x0000 
  #define SCARD_F_INTERNAL_ERROR        (SCARD_E_PREFIX|0x0001)
  #define SCARD_E_CANCELLED             (SCARD_E_PREFIX|0x0002)
  #define SCARD_E_INVALID_HANDLE        (SCARD_E_PREFIX|0x0003)
  #define SCARD_E_INVALID_PARAMETER     (SCARD_E_PREFIX|0x0004)
  #define SCARD_E_INVALID_TARGET        (SCARD_E_PREFIX|0x0005)
  #define SCARD_E_NO_MEMORY             (SCARD_E_PREFIX|0x0006)
  #define SCARD_F_WAITED_TOO_LONG       (SCARD_E_PREFIX|0x0007)
  #define SCARD_E_INSUFFICIENT_BUFFER   (SCARD_E_PREFIX|0x0008)
  #define SCARD_E_UNKNOWN_READER        (SCARD_E_PREFIX|0x0009)
  #define SCARD_E_TIMEOUT               (SCARD_E_PREFIX|0x000A)
  #define SCARD_E_SHARING_VIOLATION     (SCARD_E_PREFIX|0x000B)
  #define SCARD_E_NO_SMARTCARD          (SCARD_E_PREFIX|0x000C)
  #define SCARD_E_UNKNOWN_CARD          (SCARD_E_PREFIX|0x000D)
  #define SCARD_E_CANT_DISPOSE          (SCARD_E_PREFIX|0x000E)
  #define SCARD_E_PROTO_MISMATCH        (SCARD_E_PREFIX|0x000F)
  #define SCARD_E_NOT_READY             (SCARD_E_PREFIX|0x0010)
  #define SCARD_E_INVALID_VALUE         (SCARD_E_PREFIX|0x0011)
  #define SCARD_E_SYSTEM_CANCELLED      (SCARD_E_PREFIX|0x0012)
  #define SCARD_F_COMM_ERROR            (SCARD_E_PREFIX|0x0013)
  #define SCARD_F_UNKNOWN_ERROR         (SCARD_E_PREFIX|0x0014)
  #define SCARD_E_INVALID_ATR           (SCARD_E_PREFIX|0x0015)
  #define SCARD_E_NOT_TRANSACTED        (SCARD_E_PREFIX|0x0016)
  #define SCARD_E_READER_UNAVAILABLE    (SCARD_E_PREFIX|0x0017)
  #define SCARD_E_P_SHUTDOWN            (SCARD_E_PREFIX|0x0018)
  #define SCARD_E_PCI_TOO_SMALL         (SCARD_E_PREFIX|0x0019)
  #define SCARD_E_READER_UNSUPPORTED    (SCARD_E_PREFIX|0x001A)
  #define SCARD_E_DUPLICATE_READER      (SCARD_E_PREFIX|0x001B)
  #define SCARD_E_CARD_UNSUPPORTED      (SCARD_E_PREFIX|0x001C)
  #define SCARD_E_NO_SERVICE            (SCARD_E_PREFIX|0x001D)
  #define SCARD_E_SERVICE_STOPPED       (SCARD_E_PREFIX|0x001E)
  #define SCARD_E_UNEXPECTED            (SCARD_E_PREFIX|0x001F)
  #define SCARD_E_UNSUPPORTED_FEATURE   (SCARD_E_PREFIX|0x0022)
  #define SCARD_E_NO_READERS_AVAILABLE  (SCARD_E_PREFIX|0x002E)
  #define SCARD_E_COMM_DATA_LOST        (SCARD_E_PREFIX|0x002F)
  #define SCARD_W_UNSUPPORTED_CARD      (SCARD_E_PREFIX|0x0065)
  #define SCARD_W_UNRESPONSIVE_CARD     (SCARD_E_PREFIX|0x0066)
  #define SCARD_W_UNPOWERED_CARD        (SCARD_E_PREFIX|0x0067)
  #define SCARD_W_RESET_CARD            (SCARD_E_PREFIX|0x0068)
  #define SCARD_W_REMOVED_CARD          (SCARD_E_PREFIX|0x0069)
  #define SCARD_W_INSERTED_CARD         (SCARD_E_PREFIX|0x006A)

#endif

/*
 * Other defines
 * -------------
 */

#define SCARD_SCOPE_USER                0x0000  /* Scope in user space */
#define SCARD_SCOPE_TERMINAL            0x0001  /* Scope in terminal */
#define SCARD_SCOPE_SYSTEM              0x0002  /* Scope in system */
#define SCARD_SCOPE_GLOBAL              0x0003  /* Scope is global */

#define SCARD_PROTOCOL_T0               0x0001  /* T=0 active protocol. */
#define SCARD_PROTOCOL_T1               0x0002  /* T=1 active protocol. */
#define SCARD_PROTOCOL_Tx               0x0003  /* T=? active protocol. */
#define SCARD_PROTOCOL_RAW              0x1000  /* Raw active protocol. */
#define SCARD_PROTOCOL_PTS              0x2000  /* Force PTS */
#define SCARD_PROTOCOL_HIS              0x4000  /* T=0 or T=1 at double speed */
#define SCARD_PROTOCOL_EMV              0x8000  /* Switch to EMV compliant mode */
#define SCARD_PROTOCOL_S9               0x0008  /* S=9 active protocol */
#define SCARD_PROTOCOL_S10              0x0009  /* S=10 active protocol */

#define SCARD_SHARE_EXCLUSIVE           0x0001  /* Exclusive mode only */
#define SCARD_SHARE_SHARED              0x0002  /* Shared mode only */
#define SCARD_SHARE_DIRECT              0x0003  /* Raw mode only */

#define SCARD_LEAVE_CARD                0x0000  /* Do nothing on close */
#define SCARD_RESET_CARD                0x0001  /* Reset on close */
#define SCARD_UNPOWER_CARD              0x0002  /* Power down on close */
#define SCARD_EJECT_CARD                0x0003  /* Eject on close */

#define SCARD_UNKNOWN                   0x0001  /* Unknown state */
#define SCARD_ABSENT                    0x0002  /* Card is absent */
#define SCARD_PRESENT                   0x0004  /* Card is present */
#define SCARD_SWALLOWED                 0x0008  /* Card not powered */
#define SCARD_POWERED                   0x0010  /* Card is powered */
#define SCARD_NEGOTIABLE                0x0020  /* Ready for PTS */
#define SCARD_SPECIFIC                  0x0040  /* PTS has been set */

#define SCARD_STATE_UNAWARE             0x0000  /* App wants status */
#define SCARD_STATE_IGNORE              0x0001  /* Ignore this reader */
#define SCARD_STATE_CHANGED             0x0002  /* State has changed */
#define SCARD_STATE_UNKNOWN             0x0004  /* Reader unknown */
#define SCARD_STATE_UNAVAILABLE         0x0008  /* Status unavailable */
#define SCARD_STATE_EMPTY               0x0010  /* Card removed */
#define SCARD_STATE_PRESENT             0x0020  /* Card inserted */
#define SCARD_STATE_ATRMATCH            0x0040  /* ATR matches card */
#define SCARD_STATE_EXCLUSIVE           0x0080  /* Exclusive Mode */
#define SCARD_STATE_INUSE               0x0100  /* Shared Mode */
#define SCARD_STATE_MUTE                0x0200  /* Unresponsive card */

#define SCARD_RESET                     0x0001  /* Card was reset */
#define SCARD_INSERTED                  0x0002  /* Card was inserted */
#define SCARD_REMOVED                   0x0004  /* Card was removed */

#define BLOCK_STATUS_RESUME             0x00FF  /* Normal resume */
#define BLOCK_STATUS_BLOCKING           0x00FA  /* Function is blocking */

#define SCARD_ATR_LENGTH                MAX_ATR_SIZE  /* Maximum ATR size */

#ifdef WIN32

  /*
   * Windows specific : Unicode and Ascii support
   */
  #ifdef UNICODE
    #define SCardListReaders SCardListReadersW
    #define SCardConnect SCardConnectW
    #define SCardStatus SCardStatusW
    #define SCardGetLibrary SCardGetLibraryW

    #define SCardBinToString SCardBinToStringW
    #define SCardStringToBin SCardStringToBinW
    #define SCardHexToString SCardHexToStringW
    #define SCardStringLength SCardStringLengthW
    #define SCardErrorToString SCardErrorToStringW
    #define SCardGCRErrorToString SCardGCRErrorToStringW
    #define SCardISOErrorToString SCardISOErrorToStringW
    #define SCardMultiStringItem SCardMultiStringItemW
    #define SCardMultiStringCount SCardMultiStringCountW
    #define SCardMessageBox SCardMessageBoxW
  #else
    #define SCardListReaders SCardListReadersA
    #define SCardConnect SCardConnectA
    #define SCardStatus SCardStatusA
    #define SCardGetLibrary SCardGetLibraryA

    #define SCardBinToString SCardBinToStringA
    #define SCardStringToBin SCardStringToBinA
    #define SCardHexToString SCardHexToStringA
    #define SCardStringLength SCardStringLengthA
    #define SCardErrorToString SCardErrorToStringA
    #define SCardGCRErrorToString SCardGCRErrorToStringA
    #define SCardISOErrorToString SCardISOErrorToStringA
    #define SCardMultiStringItem SCardMultiStringItemA
    #define SCardMultiStringCount SCardMultiStringCountA
    #define SCardMessageBox SCardMessageBoxA
  #endif

  /*
   * Windows specific : export as a DLL
   */
  #ifndef SPRINGCARD_LIB
    #define SPRINGCARD_LIB __declspec( dllimport )
  #endif
  #ifndef UNDER_CE
    /* Under Desktop Windows we use the cdecl calling convention */
    #define SPRINGCARD_API __cdecl
  #else
    /* Under Windows CE we use the stdcall calling convention */
    #define SPRINGCARD_API __stdcall
  #endif

#endif

#ifdef __palmos__

  /*
   * Nothing special
   */
#define SPRINGCARD_LIB
#define SPRINGCARD_API
#define IN
#define OUT

#endif

#ifdef __linux__

  /*
   * Nothing special
   */
#define SPRINGCARD_LIB
#define SPRINGCARD_API
#define IN
#define OUT

#endif

#ifdef __cplusplus
extern  "C"
{
#endif

SPRINGCARD_LIB extern SCARD_IO_REQUEST g_rgSCardT0Pci;
SPRINGCARD_LIB extern SCARD_IO_REQUEST g_rgSCardT1Pci;
SPRINGCARD_LIB extern SCARD_IO_REQUEST g_rgSCardRawPci;
#define SCARD_PCI_T0  (&g_rgSCardT0Pci)
#define SCARD_PCI_T1  (&g_rgSCardT1Pci)
#define SCARD_PCI_RAW (&g_rgSCardRawPci)

/*
 * PC/SC compliant API
 * -------------------
 */

SPRINGCARD_LIB LONG SPRINGCARD_API SCardEstablishContext(IN DWORD dwScope,
                                                         IN LPCVOID pvReserved1,
                                                         IN LPCVOID pvReserved2,
                                                         OUT SCARD_CONTEXT_T *phContext);

SPRINGCARD_LIB LONG SPRINGCARD_API SCardReleaseContext(IN SCARD_CONTEXT_T hContext);

SPRINGCARD_LIB LONG SPRINGCARD_API SCardListReaders(IN SCARD_CONTEXT_T hContext,
                                                    IN LPCTSTR mszGroups,
                                                    OUT LPTSTR mszReaders,
                                                    IN OUT LPDWORD pcchReaders);

SPRINGCARD_LIB LONG SPRINGCARD_API SCardConnect(IN SCARD_CONTEXT_T hContext,
                                                IN LPCTSTR szReader,
                                                IN DWORD dwShareMode,
                                                IN DWORD dwPreferredProtocols,
                                                OUT SCARD_HANDLE_T *phCard,
                                                OUT LPDWORD pdwActiveProtocol);

SPRINGCARD_LIB LONG SPRINGCARD_API SCardReconnect(IN SCARD_HANDLE_T hCard,
                                                  IN DWORD dwShareMode,
                                                  IN DWORD dwPreferredProtocols,
                                                  IN DWORD dwInitialization,
                                                  OUT LPDWORD pdwActiveProtocol);

SPRINGCARD_LIB LONG SPRINGCARD_API SCardDisconnect(IN SCARD_HANDLE_T hCard,
                                                   IN DWORD dwDisposition);

SPRINGCARD_LIB LONG SPRINGCARD_API SCardBeginTransaction(IN SCARD_HANDLE_T hCard);

SPRINGCARD_LIB LONG SPRINGCARD_API SCardCancelTransaction(IN SCARD_HANDLE_T hCard);

SPRINGCARD_LIB LONG SPRINGCARD_API SCardEndTransaction(IN SCARD_HANDLE_T hCard,
                                                       IN DWORD dwDisposition);

SPRINGCARD_LIB LONG SPRINGCARD_API SCardStatus(IN SCARD_HANDLE_T hCard,
                                               OUT LPTSTR mszReaderNames,
                                               IN OUT LPDWORD pcchReaderLen,
                                               OUT LPDWORD pdwState,
                                               OUT LPDWORD pdwProtocol,
                                               OUT LPBYTE pbAtr,
                                               OUT LPDWORD pbcAtrLen);

SPRINGCARD_LIB LONG SPRINGCARD_API SCardTransmit(IN SCARD_HANDLE_T hCard,
                                                 IN LPCSCARD_IO_REQUEST pioSendPci,
                                                 IN LPBYTE pbSendBuffer,
                                                 IN DWORD cbSendLength,
                                                 IN OUT LPSCARD_IO_REQUEST pioRecvPci,
                                                 OUT LPBYTE pbRecvBuffer,
                                                 IN OUT LPDWORD pcbRecvLength);

SPRINGCARD_LIB LONG SPRINGCARD_API SCardControl(IN SCARD_HANDLE_T hCard,
                                                IN LPBYTE pbSendBuffer,
                                                IN DWORD cbSendLength,
                                                IN LPBYTE pbRecvBuffer,
                                                IN OUT LPDWORD pcbRecvLength);

SPRINGCARD_LIB LONG SPRINGCARD_API SCardSetTimeout(IN SCARD_HANDLE_T hCard,
                                                   IN DWORD dwTimeout);

/*
 * Generic utilities
 * -----------------
 */

SPRINGCARD_LIB LONG SPRINGCARD_API SCardGetLibrary(OUT TCHAR * library,
                                                   IN WORD len);

SPRINGCARD_LIB LONG SPRINGCARD_API SCardFreeMemory(IN SCARD_CONTEXT_T hContext,
                                                   IN LPVOID pvMem);

SPRINGCARD_LIB LONG SPRINGCARD_API SCardAllocMemory(IN SCARD_CONTEXT_T hContext,
                                                    IN OUT LPVOID * pvMem,
                                                    IN DWORD dwSize);

SPRINGCARD_LIB LONG SCardStringLength(IN SCARD_CONTEXT_T hContext,
                                      IN LPCTSTR szText);

SPRINGCARD_LIB LONG SPRINGCARD_API SCardStringToBin(IN SCARD_CONTEXT_T hContext,
                                                    IN OUT LPVOID pvResult,
                                                    IN DWORD dwResultLen,
                                                    IN LPCTSTR szText);

SPRINGCARD_LIB LONG SPRINGCARD_API SCardHexToString(IN SCARD_CONTEXT_T hContext,
                                                    IN LPTSTR szResult,
                                                    IN DWORD dwResultLen,
                                                    IN LPCTSTR szText);

SPRINGCARD_LIB LONG SPRINGCARD_API SCardBinToString(IN SCARD_CONTEXT_T hContext,
                                                    IN OUT LPTSTR szResult,
                                                    IN DWORD dwResultLen,
                                                    IN OUT LPVOID pvMem,
                                                    IN DWORD dwSize);

SPRINGCARD_LIB LONG SPRINGCARD_API SCardHexToByte(IN SCARD_CONTEXT_T hContext,
                                                  IN LPBYTE pbResult,
                                                  IN LPCTSTR szText);

SPRINGCARD_LIB LONG SPRINGCARD_API SCardGetLastError(IN SCARD_CONTEXT_T hContext);


SPRINGCARD_LIB LPTSTR SPRINGCARD_API SCardErrorToString(IN SCARD_CONTEXT_T hContext,
                                                        LONG dwErrCode);
SPRINGCARD_LIB LPTSTR SPRINGCARD_API SCardISOErrorToString(IN SCARD_CONTEXT_T hContext,
                                                           BYTE sw1,
                                                           BYTE sw2);

//SPRINGCARD_LIB LONG SPRINGCARD_API SCardGCRGetLastError(IN SCARD_CONTEXT_T hContext);
//SPRINGCARD_LIB LPTSTR SPRINGCARD_API SCardGCRErrorToString(IN SCARD_CONTEXT_T hContext, LONG dwErrCode);

SPRINGCARD_LIB void SPRINGCARD_API SCardMessageBox(LPCTSTR szMessage);

SPRINGCARD_LIB LPTSTR SPRINGCARD_API SCardMultiStringItem(IN SCARD_CONTEXT_T hContext,
                                                          IN LPCTSTR szMulti,
                                                          IN DWORD dwItem);

SPRINGCARD_LIB DWORD SPRINGCARD_API SCardMultiStringCount(IN SCARD_CONTEXT_T hContext,
                                                          IN LPCTSTR szMulti);
                                                          
SPRINGCARD_LIB BYTE SPRINGCARD_API SCardGetGCRFamily(void);


#ifdef __cplusplus
}
#endif

#endif
