/************************************************************/
/*                      APDU                                */
/************************************************************/
#include "stdafx.h"

#include "products/springcard/springcard.h"

#include "apdu.h"
#include "bin2c.h"

#define SCARD_APDU_BUFFER_SIZE          254

SCARDCONTEXT SCardContext;
SCARDHANDLE SCardHandle = 0;
LONG SCardLastError;
LONG SCardLastErrorDummy;
DWORD SCardProtocol;
DWORD l;
WORD SCardReaders[102];
BYTE SCardAtrBuffer[32];
WORD SCardAtrString[65];
BYTE SCardRecvBuffer[MAX_BUFFER_SIZE];
BYTE SCardSendBuffer[MAX_BUFFER_SIZE];


bool reader_ok = FALSE;
bool card_ok   = FALSE;



/*************************/
/* bool ReaderOpen(void) */
/*************************/
bool ReaderOpen(void)
{
  DWORD l;

  if (reader_ok) return TRUE;

  SCardLastError = SCardEstablishContext(SCARD_SCOPE_SYSTEM, NULL, NULL, &SCardContext);
  if (SCardLastError != SCARD_S_SUCCESS)
    return FALSE;

  l = sizeof(SCardReaders);
  SCardLastError = SCardListReaders(SCardContext, NULL, SCardReaders, &l);
  if (SCardLastError != SCARD_S_SUCCESS)
    return FALSE;

  reader_ok = TRUE;
  return TRUE;
}

/**************************/
/*    ReaderClose         */
/**************************/
void ReaderClose(void)
{
  card_ok   = FALSE;
  if (reader_ok)
    SCardReleaseContext(SCardContext);
  SCardContext = 0;
  reader_ok = FALSE;
}

/**************************/
/*       CardOpen         */
/**************************/
bool CardOpen(bool sim_card , bool mode_emv, bool pts_auto)
{
  DWORD l;
  WORD *reader_name;

  DWORD open_mode;

  if (!ReaderOpen()) return FALSE;

  if (sim_card)
    reader_name   = SCardMultiStringItem(SCardContext, SCardReaders, 1);
  else
    reader_name   = SCardMultiStringItem(SCardContext, SCardReaders, 0);

  open_mode = SCARD_PROTOCOL_Tx;
  if (mode_emv)
    open_mode |= SCARD_PROTOCOL_EMV;
  if (pts_auto)
    open_mode |= SCARD_PROTOCOL_PTS;

  SCardLastError = SCardConnect(SCardContext, reader_name, SCARD_SHARE_EXCLUSIVE, open_mode, &SCardHandle, &SCardProtocol);

  if (SCardLastError != SCARD_S_SUCCESS)
  {
    SCardDisconnect(SCardHandle,SCARD_EJECT_CARD);
    return FALSE; 
  }

  l = sizeof(SCardAtrBuffer);
  SCardLastError = SCardStatus(SCardHandle, NULL, NULL, NULL, NULL, SCardAtrBuffer, &l);
  if (SCardLastError != SCARD_S_SUCCESS)
  {
	SCardDisconnect(SCardHandle,SCARD_EJECT_CARD);
    return FALSE;
  }
//  ta1 = SCardAtrBuffer[2];
  
  SCardLastError = SCardBinToString(SCardContext, SCardAtrString, sizeof(SCardAtrString), SCardAtrBuffer, l);
  if (SCardLastError != SCARD_S_SUCCESS)
  {
    SCardDisconnect(SCardHandle,SCARD_EJECT_CARD);
    return FALSE;
  }
  /* TODO : auto PTS here !!! */

  return TRUE;
}

/**************************/
/*      CardClose         */
/**************************/
bool CardClose(void)
{
  if (SCardHandle != 0)
  {
    SCardDisconnect(SCardHandle, SCARD_UNPOWER_CARD);
    SCardHandle = 0;
  }
  return true;
}

/**************************/
/*      CardStatus        */
/**************************/
bool CardPresent(void)
{
  DWORD l;
  DWORD state;

  if (SCardHandle != 0)
  {
    l = sizeof(SCardAtrBuffer);
  
    SCardLastError = SCardStatus(SCardHandle, NULL, NULL, &state, NULL, SCardAtrBuffer, &l);
    if (SCardLastError != SCARD_S_SUCCESS)
	  return false;
    else
	{
      if (state & SCARD_ABSENT)
		return false;
	  else
		return true;
    }
  }
  return false;
}

/**************************/
bool APDU_error_display(bool sim)
{
  // TODO
  return false;
}

bool APDU_display_firmware(bool sim)
{  
  return false;
}


void APDU_pts(bool sim, BYTE ta1)
{ 
  
  /* First no pts management */
//  GCR_easy_command("1211", NULL, NULL);
  /* PTS parameter */
//  strcpy(command,"12F110");
//  strcat(command,HEX(ta1));
//  strcat(command,HEX(0x10^ta1^0xff));
//  GCR_easy_command(command, NULL, NULL);
  /* gemcore in dummy state */
//  SysTaskDelay(SysTicksPerSecond());		
//  GCR_card_power_down(sim);
 // GCR_off();
}


/**************************************/
/*          APDU_shutdown             */
/**************************************/
void APDU_shutdown(bool sim)
{
  // TODO
}
/*******************************************/
/*          APDU_set_delay                 */
/*******************************************/
bool APDU_set_delay(BYTE timeout)
{
  // TODO
  return TRUE;
}

/*******************************************/
/*          SpringCard_ApduExchange        */
/*******************************************/
bool SpringCard_ApduExchange(LPCTSTR ApduStr, DWORD ApduLen, TCHAR *ResultStr ,DWORD ResultLen)
{
	LPVOID pSCardSendBuffer;
	DWORD RecvLength;
	
	if (!reader_ok)
	{
		_tcscpy(ResultStr, _T("Reader Close"));
        return false;
	}

	if (SCardHandle ==0)
	{
		_tcscpy(ResultStr, _T("Card powerdown"));
        return false;
	}
 
    SCardLastError = SCardAllocMemory(SCardContext, &pSCardSendBuffer, ApduLen/2);

    if (SCARD_S_SUCCESS != SCardLastError)
	{
        _tcscpy(ResultStr, _T("Alloc failed"));
        return false;
    }


    SCardLastError = SCardStringToBin(SCardContext, pSCardSendBuffer, ApduLen/2, ApduStr);
    if (SCARD_S_SUCCESS != SCardLastError)
    {
        _tcscpy(ResultStr, _T("String to bin failed"));
        SCardLastErrorDummy = SCardFreeMemory(SCardContext, pSCardSendBuffer);
        return false;
    }


    // Perform APDU exchange 
    RecvLength = SCARD_APDU_BUFFER_SIZE;
    SCardLastError = SCardTransmit(SCardHandle, 0, (LPBYTE) pSCardSendBuffer, ApduLen/2, 0, SCardRecvBuffer, &RecvLength);
    if (SCARD_S_SUCCESS != SCardLastError)
	{
        _tcscpy(ResultStr,_T("Failed: "));
		_tcscat(ResultStr,(TCHAR *) ApduStr);
        SCardLastErrorDummy = SCardFreeMemory(SCardContext, pSCardSendBuffer);
        return false;
    }
   

   
	SCardLastError = SCardBinToString(SCardContext, ResultStr, ResultLen , SCardRecvBuffer, RecvLength);
	if ( SCARD_S_SUCCESS != SCardLastError )
	{
		_tcscpy(ResultStr , _T("Bin to string failed"));
		SCardLastErrorDummy = SCardFreeMemory(SCardContext, pSCardSendBuffer);
	    return false;
	 }
		
    // Free buffers
    SCardLastErrorDummy = SCardFreeMemory(SCardContext, pSCardSendBuffer);
   
    return true;
}

/***********************/
/* APDU_exchange       */
/***********************/
char *HEX(int v)
{
  static char t[3];
  bin2c(v, t, 2);
  t[2] = '\0';
  return t;
}
